import { containClass, exist, haveText, notContainClass } from '../../support/cypress.util';

export class OrganisationsEinheitenE2EComponent {
  private readonly organisationsEinheitenList: string = 'organisations-einheit-list';
  private readonly organisationsEinheitenName: string = 'organisations-einheit-name';
  private readonly organisationsEinheitenID: string = 'organisations-einheit-id';
  private readonly errorColor: string = 'text-red-500';
  private readonly errorIcon: string = 'organisations-einheit-sync-error';

  private readonly errorText: string = 'Organisationseinheit wurde nicht in den PVOG-Daten gefunden.';

  public getOrganisationsEinheitList(): Cypress.Chainable<Element> {
    return cy.getTestElement(this.organisationsEinheitenList);
  }

  public getListItemByName(name: string): Cypress.Chainable<JQuery<HTMLElement>> {
    return cy.getTestElement(this.organisationsEinheitenName).contains(name);
  }

  public openOrganisationsEinheit(name: string): void {
    this.getListItemByName(name).click();
  }

  public organisationsEinheitContainsID(name: string, id: string): void {
    this.getListItemByName(name)
      .parents('a')
      .within(() => {
        haveText(cy.getTestElement(this.organisationsEinheitenID), id);
      });
  }

  public elementIsShownAsError(name: string): void {
    containClass(this.getListItemByName(name).closest('ods-list-item'), this.errorColor);
  }

  public getErrorIconInElement(name: string): Cypress.Chainable<JQuery<HTMLAnchorElement>> {
    return this.getListItemByName(name)
      .parents('a')
      .within(() => {
        exist(cy.getTestElement(this.errorIcon));
      });
  }

  public getErrorTooltip(name: string): Cypress.Chainable<JQuery<HTMLElement>> {
    return this.getErrorIconInElement(name)
      .find('ods-exclamation-icon[mattooltip]')
      .should('have.attr', 'mattooltip')
      .and('include', this.errorText);
  }

  public elementShowsNoError(name: string): void {
    notContainClass(this.getListItemByName(name).closest('ods-list-item'), this.errorColor);
  }
}
