import { MainPage } from 'apps/admin-e2e/src/page-objects/main.po';
import { BenutzerE2EComponent } from '../../../components/benutzer/benutzer.e2e.component';
import { beChecked, beEnabled, exist, notBeChecked, notBeEnabled } from '../../../support/cypress.util';
import { AlfaRollen, AlfaUsers, loginAsAriane } from '../../../support/user-util';

const mainPage: MainPage = new MainPage();
const benutzerPage: BenutzerE2EComponent = new BenutzerE2EComponent();
const organistationsEinheitOrdnungsamt: string = 'Ordnungsamt';
const organisationsEinheitDenkmalpflege: string = 'Denkmalpflege';
const organisationsEinheitWirtschaftsfoerderung: string = 'Wirtschaftsförderung';
const organisationsEinheitNone: string = 'keine zuständige Stelle zugewiesen';
const emailAddress: string = 'peter.von.der.post@ozg-sh.de';

describe('Benutzer und Rollen', () => {
  before(() => {
    loginAsAriane();
  });

  it('should open Benutzer tab and show Hinzufuegen button', () => {
    mainPage.clickBenutzerTab();

    exist(benutzerPage.getHinzufuegenButton());
  });

  it('should show users and attributes in table', () => {
    exist(benutzerPage.getUserEntry('ariane'));
    benutzerPage.stringExistsInUserEntry(AlfaRollen.USER, AlfaUsers.DOROTHEA);
    benutzerPage.stringExistsInUserEntry(organistationsEinheitOrdnungsamt, AlfaUsers.LUDWIG);
    benutzerPage.stringExistsInUserEntry(AlfaRollen.USER, AlfaUsers.ZELDA);
    benutzerPage.stringExistsInUserEntry(AlfaRollen.LOESCHEN, AlfaUsers.LUDWIG);
    benutzerPage.stringExistsInUserEntry(organisationsEinheitDenkmalpflege, AlfaUsers.ZELDA);
    benutzerPage.stringExistsInUserEntry(organisationsEinheitWirtschaftsfoerderung, AlfaUsers.ZELDA);
    benutzerPage.stringExistsInUserEntry(organisationsEinheitNone, AlfaUsers.RICHARD);
    benutzerPage.stringExistsInUserEntry(emailAddress, AlfaUsers.PETER);
    benutzerPage.stringExistsInUserEntry(AlfaRollen.POSTSTELLE, AlfaUsers.PETER);
  });

  it('should show single user screen on click', () => {
    benutzerPage.addUser();

    exist(benutzerPage.getVornameInput());
    exist(benutzerPage.getNachnameInput());
    exist(benutzerPage.getBenutzernameInput());
    exist(benutzerPage.getMailInput());

    notBeChecked(benutzerPage.getAdminCheckbox());
    notBeChecked(benutzerPage.getLoeschenCheckbox());
    notBeChecked(benutzerPage.getUserCheckbox());
    notBeChecked(benutzerPage.getPostCheckbox());
  });

  it('should activate loeschen checkbox and deactivate the other two checkboxes', () => {
    benutzerPage.clickLoeschenCheckbox();
    beChecked(benutzerPage.getLoeschenCheckbox());
    notBeEnabled(benutzerPage.getUserCheckbox());
    notBeEnabled(benutzerPage.getPostCheckbox());

    benutzerPage.clickLoeschenCheckbox();
    notBeChecked(benutzerPage.getLoeschenCheckbox());
    beEnabled(benutzerPage.getUserCheckbox());
    beEnabled(benutzerPage.getPostCheckbox());
  });

  it('should additionally activate and deactivate admin checkbox', () => {
    benutzerPage.clickLoeschenCheckbox();
    benutzerPage.clickAdminCheckbox();
    beChecked(benutzerPage.getLoeschenCheckbox());
    beChecked(benutzerPage.getAdminCheckbox());

    benutzerPage.clickAdminCheckbox();
    notBeChecked(benutzerPage.getAdminCheckbox());
  });

  it('should activate user checkbox and deactivate the other two checkboxes', () => {
    benutzerPage.clickLoeschenCheckbox();
    benutzerPage.clickUserCheckbox();
    beChecked(benutzerPage.getUserCheckbox());
    notBeEnabled(benutzerPage.getLoeschenCheckbox());
    notBeEnabled(benutzerPage.getPostCheckbox());

    benutzerPage.clickUserCheckbox();
    notBeChecked(benutzerPage.getUserCheckbox());
    beEnabled(benutzerPage.getLoeschenCheckbox());
    beEnabled(benutzerPage.getPostCheckbox());
  });

  it('should activate post checkbox and deactivate the other two checkboxes', () => {
    benutzerPage.clickPostCheckbox();
    beChecked(benutzerPage.getPostCheckbox());
    notBeEnabled(benutzerPage.getLoeschenCheckbox());
    notBeEnabled(benutzerPage.getUserCheckbox());

    benutzerPage.clickPostCheckbox();
    notBeChecked(benutzerPage.getPostCheckbox());
    beEnabled(benutzerPage.getLoeschenCheckbox());
    beEnabled(benutzerPage.getUserCheckbox());
  });
});
