import { OrganisationsEinheitenE2EComponent } from '../../../components/organisationseinheiten/organisationseinheiten.e2e.component';
import { MainPage, waitForSpinnerToDisappear } from '../../../page-objects/main.po';
import { exist } from '../../../support/cypress.util';
import { OrganisationsEinheitSyncResultE2E } from '../../../support/organisationseinheit';
import {
  createBauamtOrganisationsEinheit,
  createDenkmalpflegeOrganisationsEinheit,
  createFundstelleOrganisationsEinheit,
  initOrganisationsEinheiten,
} from '../../../support/organisationseinheit-util';
import { loginAsAriane } from '../../../support/user-util';

describe('show Organisationsheiten', () => {
  const mainPage: MainPage = new MainPage();
  const organisationsEinheitenTab: OrganisationsEinheitenE2EComponent = new OrganisationsEinheitenE2EComponent();

  before(() => {
    loginAsAriane();
    initOrganisationsEinheiten([
      createBauamtOrganisationsEinheit(),
      { ...createDenkmalpflegeOrganisationsEinheit(), syncResult: OrganisationsEinheitSyncResultE2E.NAME_MISMATCH },
      { ...createFundstelleOrganisationsEinheit(), syncResult: OrganisationsEinheitSyncResultE2E.NOT_FOUND_IN_PVOG },
    ]);
  });

  it('should show table of Organisationseinheiten', () => {
    waitForSpinnerToDisappear();
    mainPage.openOrganisationsEinheiten();

    exist(organisationsEinheitenTab.getOrganisationsEinheitList());
  });

  it('should show identical data in Keycloak and PVOG without error', () => {
    organisationsEinheitenTab.getListItemByName('Bauamt');
    organisationsEinheitenTab.organisationsEinheitContainsID('Bauamt', '248240886');
    organisationsEinheitenTab.elementShowsNoError('Bauamt');
  });

  it('should show data not found in PVOG in red color and with error icon and tooltip message', () => {
    const organisationsEinheitName: string = 'Fundstelle';

    organisationsEinheitenTab.organisationsEinheitContainsID(organisationsEinheitName, '10363455');
    organisationsEinheitenTab.elementIsShownAsError(organisationsEinheitName);
    organisationsEinheitenTab.getErrorIconInElement(organisationsEinheitName);
    organisationsEinheitenTab.getErrorTooltip(organisationsEinheitName);
  });

  it('should get name for OE from PVOG, if mismatch with Keycloak, but without error', () => {
    organisationsEinheitenTab.organisationsEinheitContainsID('Landesamt für Denkmalpflege', '9093371');
    organisationsEinheitenTab.elementShowsNoError('Landesamt für Denkmalpflege');
  });
});
