/// <reference types="cypress" />

enum HttpMethod {
  POST = 'POST',
  GET = 'GET',
}

const DATA_TEST_ID: string = 'data-test-id';
const DATA_TEST_CLASS: string = 'data-test-class';

const ACCES_TOKEN: string = 'access_token';

enum Header {
  CONTENT_TYPE = 'Content-Type',
  AUTHORIZATION = 'Authorization',
}

enum CypressEnv {
  KEYCLOAK_CLIENT = 'keycloakClient',
  KEYCLOAK_REALM = 'keycloakRealm',
  KEYCLOAK_URL = 'keycloakUrl',
  SEARCH = 'search',
  SMOCKER = 'smocker',
}

Cypress.Commands.add('getTestElement', (selector, ...args) => {
  return cy.get(`[${DATA_TEST_ID}~="${selector}"]`, ...args);
});

Cypress.Commands.add('getTestElementWithClass', (selector, ...args) => {
  console.log(
    'Achtung: Potentiell nicht eindeutiges Ergebnis, weil eine data-test-class mit cy.get() von der DOM-Root aus gesucht wird.',
  );
  return cy.get(`[${DATA_TEST_CLASS}="${selector}"]`, ...args);
});

Cypress.Commands.add('getTestElementWithOid', (oid, ...args) => {
  return cy.getTestElement(oid, ...args);
});

Cypress.Commands.add(
  'findTestElementWithClass',
  { prevSubject: true },
  (subject: any, selector) => {
    return subject.find(`[${DATA_TEST_CLASS}="${selector}"]`);
  },
);

Cypress.Commands.add('findElement', { prevSubject: true }, (subject: any, selector: string) => {
  return subject.find(selector);
});

Cypress.Commands.add('getUserInfo', () => {
  return cy.request({
    method: HttpMethod.GET,
    url: `${getKeycloakBaseRealmUrl()}/userinfo`,
    headers: {
      [Header.AUTHORIZATION]: `bearer ${window.sessionStorage.getItem(ACCES_TOKEN)}`,
    },
  });
});

function getKeycloakBaseRealmUrl(): string {
  return `${Cypress.env(CypressEnv.KEYCLOAK_URL)}realms/${Cypress.env(CypressEnv.KEYCLOAK_REALM)}/protocol/openid-connect`;
}

type GetOptions = Partial<
  Cypress.Loggable & Cypress.Timeoutable & Cypress.Withinable & Cypress.Shadow
>;

declare global {
  // eslint-disable-next-line
  namespace Cypress {
    interface Chainable {
      getTestElement(selector: string, ...args: GetOptions[]): Chainable<Element>;
      getTestElementWithClass(selector: string, ...args: GetOptions[]): Chainable<Element>;
      getTestElementWithOid(oid: string, ...args: GetOptions[]): Chainable<Element>;
      findTestElementWithClass(selector: string): Chainable<Element>;
      findElement(selector: string): Chainable<Element>;
      getUserInfo(): Chainable<void>;
    }
  }
}
