import { wait } from './cypress-helper';

//TODO Naming der Methoden geradeziehen

export function containClass(element: Cypress.Chainable<Element>, cssClass: string): void {
  element.should('have.class', cssClass);
}

export function notContainClass(element: Cypress.Chainable<Element>, cssClass: string): void {
  element.should('not.have.class', cssClass);
}

export function exist(element: Cypress.Chainable<Element>): void {
  element.should('exist');
}

export function notExist(element: Cypress.Chainable<Element>): void {
  element.should('not.exist');
}

export function haveText(element: Cypress.Chainable<Element>, text: string): void {
  element
    .invoke('text')
    .then((elementText) => elementText.trim())
    .should('equal', text);
}

export function haveValue(element: Cypress.Chainable<Element>, value: string): void {
  element.should('have.value', value);
}

export function haveFocus(element: Cypress.Chainable<Element>): void {
  element.should('have.focus');
}

export function mouseEnter(element: Cypress.Chainable<Element>): void {
  element.trigger('mouseenter');
}

export function mouseOver(element: Cypress.Chainable<Element>): void {
  element.trigger('mouseover');
}

export function contains(element: Cypress.Chainable<Element>, containing: string): void {
  element.should('exist').contains(containing);
}

export function notContains(element: Cypress.Chainable<Element>, containing: string): void {
  element.contains(containing).should('not.exist');
}

export function haveLength(element: Cypress.Chainable<Element>, length: number): void {
  element.should('have.length', length);
}

export function beChecked(element: Cypress.Chainable<Element>): void {
  element.should('be.checked');
}

export function notBeChecked(element: Cypress.Chainable<Element>): void {
  element.should('not.be.checked');
}

export function beEnabled(element: Cypress.Chainable<Element>): void {
  element.should('be.enabled');
}

export function notBeEnabled(element: Cypress.Chainable<Element>): void {
  element.should('not.be.enabled');
}

//TODO: "first()" rausnehmen -> im html eine entprechende data-test-id ansprechen?! | trennen in "get" und "verify"
export function shouldFirstContains(element: Cypress.Chainable<Element>, containing: string) {
  element.first().should('exist').contains(containing);
}

export function shouldHaveAttributeBeGreaterThan(element: Cypress.Chainable<Element>, attributeName: string, value: number) {
  element.first().should('exist').invoke(attributeName).should('be.gt', value);
}

export function shouldHaveAttributeBeLowerThan(element: Cypress.Chainable<Element>, attributeName: string, value: number) {
  element.first().should('exist').invoke(attributeName).should('be.gt', value);
}
//

export function shouldHaveAttribute(element: Cypress.Chainable<Element>, name: string, value: string) {
  element.should('have.attr', name, value);
}

export function visible(element: Cypress.Chainable<Element>) {
  element.should('be.visible');
}

export function notBeVisible(element: Cypress.Chainable<Element>) {
  element.should('not.be.visible');
}

export function enter(element: Cypress.Chainable<Element>): void {
  element.clear().type(CypressKeyboardActions.ENTER);
}

export function enterWith(element: Cypress.Chainable<JQuery<HTMLElement>>, value: string, delayBeforeEnter: number = 200): void {
  element.clear().type(value);
  wait(delayBeforeEnter);
  element.type(CypressKeyboardActions.ENTER);
}

export function typeText(element: Cypress.Chainable<JQuery<HTMLElement>>, value: string): void {
  element.type(value);
}

export function backspaceOn(element: Cypress.Chainable<Element>): void {
  element.type(CypressKeyboardActions.BACKSPACE);
}

export function clearText(element: Cypress.Chainable<Element>): void {
  element.type('{selectAll}{del}');
}

enum CypressKeyboardActions {
  ENTER = '{enter}',
  BACKSPACE = '{backspace}',
}
