import { UserE2E } from '../model/user';
import { HeaderE2EComponent } from '../page-objects/header.po';
import { MainPage } from '../page-objects/main.po';

export function loginAsAriane(): void {
  cy.fixture('user/user_ariane.json').then((user) => {
    loginByUi(user);
  });
}

export function loginAsSabine(): void {
  cy.fixture('user/user_sabine.json').then((user) => {
    loginByUi(user);
  });
}

// Hinweis: cacheAcrossSpecs: true lässt Tests umfallen
export function loginByUi(user: UserE2E): void {
  cy.session(
    user.name,
    () => {
      cy.visit('/')
        .get('#kc-login')
        .should('exist')
        .then(() => {
          cy.get('#username').type(user.name);
          cy.get('#password').type(user.password);
          cy.get('#kc-login').click();
        });
    },
    {
      cacheAcrossSpecs: false,
      validate() {
        cy.window().its('sessionStorage').invoke('getItem', 'access_token').should('not.be.empty');
      },
    },
  );
}

export function logout(): void {
  const mainPage: MainPage = new MainPage();
  const header: HeaderE2EComponent = mainPage.getHeader();
  header.getCurrentUserProfile().logout();
}

export enum AlfaRollen {
  USER = 'VERWALTUNG_USER',
  LOESCHEN = 'VERWALTUNG_LOESCHEN',
  POSTSTELLE = 'VERWALTUNG_POSTSTELLE',
}

export enum AlfaUsers {
  DOROTHEA = 'dorothea',
  LUDWIG = 'ludwig',
  PETER = 'peter',
  RICHARD = 'richard',
  ZELDA = 'zelda',
}
