/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ('Lizenz');
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software 'so wie sie
 * ist', OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { CurrentUserProfileE2EComponent } from 'apps/alfa-e2e/src/components/user-profile/current-user-profile.component.e2e';
import { UserE2E } from 'apps/alfa-e2e/src/model/user';
import { HeaderE2EComponent } from 'apps/alfa-e2e/src/page-objects/header.po';
import {
  MainPage,
  waitForSpinnerToDisappear,
  waitforSpinnerToAppear,
} from 'apps/alfa-e2e/src/page-objects/main.po';
import { dropCollections } from 'apps/alfa-e2e/src/support/cypress-helper';
import { exist, haveText } from 'apps/alfa-e2e/src/support/cypress.util';
import {
  getUserSabine,
  initUsermanagerUsers,
  loginAsSabine,
} from 'apps/alfa-e2e/src/support/user-util';

describe('Current User Profile', () => {
  const mainPage: MainPage = new MainPage();

  const header: HeaderE2EComponent = mainPage.getHeader();
  const currentUserProfile: CurrentUserProfileE2EComponent = header.getCurrentUserProfile();

  const userSabine: UserE2E = getUserSabine();

  describe('for sabine', () => {
    before(() => {
      initUsermanagerUsers();

      loginAsSabine();

      waitforSpinnerToAppear();
      waitForSpinnerToDisappear();
      exist(header.getRoot());
    });

    after(() => {
      dropCollections();
    });

    it('should show current user profile', () => {
      exist(currentUserProfile.getRoot());
    });

    it('should show assigned icon', () => {
      exist(currentUserProfile.getUserProfile().getIconContainer().getAssignedIcon());
    });

    it('should show initials', () => {
      haveText(
        currentUserProfile.getUserProfile().getIconContainer().getAssignedIcon(),
        userSabine.initials,
      );
    });
  });
});
