/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { registerLocaleData } from '@angular/common';
import localeDe from '@angular/common/locales/de';
import localeDeExtra from '@angular/common/locales/extra/de';
import { VorgangListE2EComponent } from '../../../components/vorgang/vorgang-list.e2e.component';
import { VorgangE2E } from '../../../model/vorgang';
import { MainPage, waitForSpinnerToDisappear } from '../../../page-objects/main.po';
import { VorgangPage } from '../../../page-objects/vorgang.po';
import { WiedervorlagePage } from '../../../page-objects/wiedervorlage.po';
import { dropCollections } from '../../../support/cypress-helper';
import { exist } from '../../../support/cypress.util';
import { loginAsSabine } from '../../../support/user-util';
import { createVorgang, initVorgang } from '../../../support/vorgang-util';

registerLocaleData(localeDe, 'de', localeDeExtra);

describe('Vorgang Detailansicht Wiedervorlage routing', () => {
  const mainPage: MainPage = new MainPage();
  const vorgangList: VorgangListE2EComponent = mainPage.getVorgangList();

  const vorgangPage: VorgangPage = new VorgangPage();
  const wiedervorlagePage: WiedervorlagePage = new WiedervorlagePage();

  const vorgang: VorgangE2E = createVorgang();

  before(() => {
    initVorgang(vorgang);

    loginAsSabine();

    waitForSpinnerToDisappear();
    exist(vorgangList.getRoot());
  });

  after(() => {
    dropCollections();
  });

  it('Open Vorgang-Detail-Page', () => {
    vorgangList.getListItem(vorgang.name).getRoot().click();
    waitForSpinnerToDisappear();

    exist(vorgangPage.getVorgangDetailHeader().getRoot());
  });

  it('click on create wiedervorlage icon button should open page', () => {
    vorgangPage.getSubnavigation().getWiedervorlageIconButton().click();
    waitForSpinnerToDisappear();

    exist(wiedervorlagePage.getSubnavigation().getRoot());
  });

  it('click on navigate back', () => {
    wiedervorlagePage.getSubnavigation().navigateBack();
    waitForSpinnerToDisappear();

    exist(vorgangPage.getVorgangDetailHeader().getRoot());
  });

  it('click on create wiedervorlage button should open page', () => {
    vorgangPage.getWiedervorlagenContainer().getCreateWiedervorlageButton().click();
    waitForSpinnerToDisappear();

    exist(wiedervorlagePage.getSubnavigation().getRoot());
  });

  it('click on navigate back', () => {
    wiedervorlagePage.getSubnavigation().navigateBack();
    waitForSpinnerToDisappear();

    exist(vorgangPage.getVorgangDetailHeader().getRoot());
  });
});
