/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { encodeUrlForEmbedding } from '@alfa-client/tech-shared';
import { isUndefined } from 'lodash-es';
import {
  AccessPermissionE2E,
  AntragstellerE2E,
  ClientAttributeE2E,
  ClientAttributesE2E,
  EingangE2E,
  EingangHeaderE2E,
  VorgangE2E,
} from '../model/vorgang';
import { VorgangAttachedItemClientE2E } from '../model/vorgang-attached-item';
import { initSearchIndexData, initVorgangData } from './cypress-helper';
import { buildUrl } from './tech.util';

export const objectIds: string[] = require('../fixtures/util/object_id_collection.json');
const vorgangFixture: VorgangE2E = require('../fixtures/vorgang/vorgang.json');

export function createVorgang(): VorgangE2E {
  return vorgangFixture;
}

export function buildVorgang($oid: string, uiIdentifier: string): VorgangE2E {
  return {
    ...createVorgang(),
    _id: { $oid },
    name: uiIdentifier,
  };
}

export function initSearchIndex(vorgaenge: VorgangE2E[]): void {
  initSearchIndexData(vorgaenge);
}

export function initVorgang(vorgang: VorgangE2E): void {
  initVorgaenge([vorgang]);
}

export function initVorgaenge(vorgaenge: VorgangE2E[]): void {
  initVorgangData(vorgaenge);
}

export const EingangE2ETestData: EingangE2E = createVorgang().eingangs[0];
export const AntragstellerE2ETestData: AntragstellerE2E = EingangE2ETestData.antragsteller;
export const EingangHeaderE2ETestData: EingangHeaderE2E = EingangE2ETestData.header;

export function createVorgangUriById(id: string): string {
  return `/vorgang/${encodeUrlForEmbedding(buildUrl(id, 'vorgangs'))}`;
}

export const VORGANG_LIST_PAGE_SIZE: number = 100;
export const VORGANG_LIST_NEXT_FRIST_PARAM: string = 'nextFrist=exists';

export function buildVorgangListPageLink(page: number, assignedTo?: string): string {
  let link: string = `vorgangs?page=${page - 1}&limit=100`;

  if (isUndefined(assignedTo)) {
    return link;
  }

  return `${link}&assignedTo=${assignedTo}`;
}

//Client Attribute

const CLIENT_ATTRIBUTE_CLASS: string = 'ClientAttribute';

export function createAlfaClientAttributes(
  attributeName: string,
  clientAttribute: ClientAttributeE2E,
): ClientAttributesE2E {
  return { [VorgangAttachedItemClientE2E.ALFA]: { [attributeName]: clientAttribute } };
}

export function createNextWiedervorlageFristClientAttribute(value: string): ClientAttributeE2E {
  return createReadOnlyClientAttribute(value);
}

export function createHasPostfachNachrichtClientAttribute(value: boolean): ClientAttributeE2E {
  return createReadOnlyClientAttribute(value);
}

export function createHasNewPostfachNachrichtClientAttribute(value: boolean): ClientAttributeE2E {
  return <ClientAttributeE2E>{
    access: AccessPermissionE2E.READ_WRITE,
    value,
    _class: CLIENT_ATTRIBUTE_CLASS,
  };
}

function createReadOnlyClientAttribute(value: string | boolean): ClientAttributeE2E {
  return <ClientAttributeE2E>{
    access: AccessPermissionE2E.READ_ONLY,
    value,
    _class: CLIENT_ATTRIBUTE_CLASS,
  };
}
