import { StateResource, createStateResource } from '@alfa-client/tech-shared';
import { Mock, mock, useFromMock } from '@alfa-client/test-utils';
import { singleCold, singleHot } from 'libs/tech-shared/test/marbles';
import { Observable } from 'rxjs';
import { createConfigurationResource } from '../../../test/configuration/configuration';
import { ConfigurationResourceService } from './configuration-resource.service';
import { ConfigurationResource } from './configuration.model';
import { ConfigurationService } from './configuration.service';

describe('ConfigurationService', () => {
  let service: ConfigurationService;
  let configurationResourceService: Mock<ConfigurationResourceService>;

  beforeEach(() => {
    configurationResourceService = mock(ConfigurationResourceService);

    service = new ConfigurationService(useFromMock(configurationResourceService));
  });

  it('should create', () => {
    expect(service).toBeTruthy();
  });

  describe('get', () => {
    const configurationResource: ConfigurationResource = createConfigurationResource();
    const configurationStateResource: StateResource<ConfigurationResource> =
      createStateResource(configurationResource);

    beforeEach(() => {
      configurationResourceService.get = jest
        .fn()
        .mockReturnValue(singleHot(configurationStateResource));
    });
    it('should call resourceService', () => {
      configurationResourceService.get = jest.fn();

      service.get();

      expect(configurationResourceService.get).toHaveBeenCalled();
    });

    it('should return value', () => {
      const loadedConfigurationResource: Observable<StateResource<ConfigurationResource>> =
        service.get();

      expect(loadedConfigurationResource).toBeObservable(singleCold(configurationStateResource));
    });
  });
});
