import {
  AdminOrganisationsEinheitListResource,
  AdminOrganisationsEinheitResource,
  AdminOrganisationsEinheitSettings,
} from '@admin-client/admin-settings';
import { NavigationService } from '@alfa-client/navigation-shared';
import { StateResource, createEmptyStateResource, isNotUndefined } from '@alfa-client/tech-shared';
import { SnackBarService } from '@alfa-client/ui';
import { Injectable } from '@angular/core';
import { Params } from '@angular/router';
import { ResourceUri } from '@ngxp/rest';
import { isNil } from 'lodash-es';
import { Observable, Subscription, startWith, tap } from 'rxjs';
import { OrganisationsEinheitListResourceService } from './organisations-einheit-list-resource.service';
import { OrganisationsEinheitResourceService } from './organisations-einheit-resource.service';

@Injectable({
  providedIn: 'root',
})
export class OrganisationsEinheitService {
  static ORGANISATIONS_EINHEIT_URL: string = 'organisationsEinheitUrl';

  private subscription: Subscription;

  constructor(
    private listResourceService: OrganisationsEinheitListResourceService,
    private resourceService: OrganisationsEinheitResourceService,
    private navigationService: NavigationService,
    private snackBarService: SnackBarService,
  ) {
    this.listenToNavigation();
  }

  private listenToNavigation(): void {
    this.unsubscribe();
    this.subscription = this.navigationService.urlChanged().subscribe((params: Params) => this.onNavigation(params));
  }

  private unsubscribe(): void {
    if (!isNil(this.subscription)) {
      this.subscription.unsubscribe();
    }
  }

  onNavigation(params: Params): void {
    if (this.navigateToOrganisationsEinheitDetailPage(params)) {
      this.listResourceService.select(this.getOrganisationsEinheitUrl());
    }
  }

  private navigateToOrganisationsEinheitDetailPage(params: Params): boolean {
    return isNotUndefined(params[OrganisationsEinheitService.ORGANISATIONS_EINHEIT_URL]);
  }

  getOrganisationsEinheitUrl(): ResourceUri {
    return this.navigationService.getDecodedParam(OrganisationsEinheitService.ORGANISATIONS_EINHEIT_URL);
  }

  public getList(): Observable<StateResource<AdminOrganisationsEinheitListResource>> {
    return this.listResourceService.getList();
  }

  public get(): Observable<StateResource<AdminOrganisationsEinheitResource>> {
    return this.resourceService.get();
  }

  public patch(
    organisationsEinheitSettings: AdminOrganisationsEinheitSettings,
  ): Observable<StateResource<AdminOrganisationsEinheitResource>> {
    return this.resourceService.patch(organisationsEinheitSettings).pipe(
      tap((stateResource: StateResource<AdminOrganisationsEinheitResource>) => this.showInfoAfterPatch(stateResource)),
      startWith(createEmptyStateResource<AdminOrganisationsEinheitResource>(true)),
    );
  }

  private showInfoAfterPatch(stateResource: StateResource<AdminOrganisationsEinheitResource>) {
    if (!stateResource.loading) {
      this.snackBarService.showInfo('Die Signatur wurde erfolgreich gespeichert.');
    }
  }
}
