import { mock, Mock, useFromMock } from '@alfa-client/test-utils';
import { fakeAsync, tick } from '@angular/core/testing';
import { of } from 'rxjs';
import { createUser } from '../../../test/user/user';
import { User } from '../user/user.model';
import { UserRepository } from '../user/user.repository.service';
import * as UserUtil from '../user/user.util';
import { UserService } from './user.service';

describe('UserService', () => {
  let service: UserService;
  let repository: Mock<UserRepository>;

  const user: User = createUser();

  beforeEach(() => {
    repository = { ...mock(UserRepository), getUsers: jest.fn().mockReturnValue(of([user])) };
    service = new UserService(useFromMock(repository));
  });

  describe('getItemsFromKeycloak', () => {
    it('should call getUsers from userRepository', () => {
      service.getItemsFromKeycloak();

      expect(repository.getUsers).toHaveBeenCalled();
    });

    it('should call sortUsersByLastName', fakeAsync(() => {
      const sortUsersByLastNameSpy: jest.SpyInstance<User[]> = jest.spyOn(UserUtil, 'sortUsersByLastName');

      service.getItemsFromKeycloak().subscribe();
      tick();

      expect(sortUsersByLastNameSpy).toHaveBeenCalled();
    }));
  });
});
