import { createStateResource, StateResource } from '@alfa-client/tech-shared';
import { faker } from '@faker-js/faker';
import { toResource } from 'libs/tech-shared/test/resource';
import { times } from 'lodash-es';
import { createApiError } from '../../../tech-shared/test/error';
import { BescheidListLinkRel } from '../lib/bescheid.linkrel';
import {
  Bescheid,
  BescheidListResource,
  BescheidResource,
  BescheidSendBy,
  BescheidStatus,
  UploadFileInProgress,
} from '../lib/bescheid.model';

export function createBescheid(): Bescheid {
  return {
    status: BescheidStatus.DRAFT,
    beschiedenAm: '2024-01-01',
    bewilligt: true,
    bescheidDocument: faker.internet.url(),
    attachments: [faker.internet.url(), faker.internet.url()],
    sendBy: BescheidSendBy.NACHRICHT,
    nachrichtSubject: faker.lorem.text(),
    nachrichtText: faker.lorem.text(),
  };
}

export function createBescheidResource(linkRel: string[] = []): BescheidResource {
  return toResource(createBescheid(), linkRel);
}

export function createBescheidStateResource(
  linkRel: string[] = [],
): StateResource<BescheidResource> {
  return createStateResource(createBescheidResource(linkRel));
}

export function createUploadFileInProgress(): UploadFileInProgress {
  return { loading: true, fileName: faker.person.firstName(), error: createApiError() };
}

export function createBescheidResources(linkRelations: string[] = []): BescheidResource[] {
  return times(10, () => toResource(createBescheidResource(), [...linkRelations]));
}

export function createBescheidListResource(
  bescheide: BescheidResource[] = createBescheidResources(),
  linkRelations: string[] = [],
): BescheidListResource {
  return toResource({}, [...linkRelations], {
    [BescheidListLinkRel.BESCHEID_LIST]: bescheide,
  });
}
