import { BescheidResource } from '@alfa-client/bescheid-shared';
import { BinaryFile2ContainerComponent, BinaryFileContainerComponent } from '@alfa-client/binary-file';
import { BinaryFileResource } from '@alfa-client/binary-file-shared';
import { StateResource, createStateResource } from '@alfa-client/tech-shared';
import { existsAsHtmlElement, notExistsAsHtmlElement } from '@alfa-client/test-utils';
import { ExpansionPanelComponent, SpinnerComponent } from '@alfa-client/ui';
import { registerLocaleData } from '@angular/common';
import localeDe from '@angular/common/locales/de';
import { ComponentFixture, TestBed } from '@angular/core/testing';
import { FormatFullDatePipe } from 'libs/tech-shared/src/lib/pipe/format-full-date.pipe';
import { MockComponent } from 'ng-mocks';
import { createBescheidResource } from '../../../../../bescheid-shared/src/test/bescheid';
import { createBinaryFileResource } from '../../../../../binary-file-shared/test/binary-file';
import { getDataTestIdOf } from '../../../../../tech-shared/test/data-test';
import { BescheidInVorgangComponent } from './bescheid-in-vorgang.component';

import { BescheidStatusTextComponent } from '@ods/system';

registerLocaleData(localeDe);

describe('BescheidInVorgangComponent', () => {
  let component: BescheidInVorgangComponent;
  let fixture: ComponentFixture<BescheidInVorgangComponent>;

  const bescheidResource: BescheidResource = createBescheidResource();
  const bescheidStateResource: StateResource<BescheidResource> = createStateResource(bescheidResource);

  const binaryFile: BinaryFileResource = createBinaryFileResource();
  const binaryFileStateResource: StateResource<BinaryFileResource> = createStateResource(binaryFile);

  const bescheidDocumentBinaryFileContainer: string = getDataTestIdOf('bescheid-document-in-vorgang-binary-file');

  beforeEach(async () => {
    await TestBed.configureTestingModule({
      declarations: [
        BescheidInVorgangComponent,
        FormatFullDatePipe,
        MockComponent(SpinnerComponent),
        MockComponent(ExpansionPanelComponent),
        MockComponent(BinaryFileContainerComponent),
        MockComponent(BescheidStatusTextComponent),
        MockComponent(BinaryFile2ContainerComponent),
      ],
    }).compileComponents();

    fixture = TestBed.createComponent(BescheidInVorgangComponent);
    component = fixture.componentInstance;
    component.bescheidStateResource = bescheidStateResource;
    component.bescheidDocumentFile = binaryFileStateResource;
    fixture.detectChanges();
  });

  it('should create', () => {
    expect(component).toBeTruthy();
  });

  describe('render', () => {
    it('should show bescheid document file', () => {
      component.bescheidDocumentFile = createStateResource(createBinaryFileResource());

      fixture.detectChanges();

      existsAsHtmlElement(fixture, bescheidDocumentBinaryFileContainer);
    });

    it('should NOT show bescheid document file if resource null', () => {
      component.bescheidDocumentFile = { ...createStateResource(null), loaded: true };

      fixture.detectChanges();

      notExistsAsHtmlElement(fixture, bescheidDocumentBinaryFileContainer);
    });

    it('should NOT show bescheid document file if resource loading', () => {
      component.bescheidDocumentFile = {
        ...createStateResource(createBinaryFileResource()),
        loaded: false,
      };

      fixture.detectChanges();

      notExistsAsHtmlElement(fixture, bescheidDocumentBinaryFileContainer);
    });
  });

  describe('bescheidBewilligt', () => {
    it('should return true', () => {
      component.bescheidStateResource.resource.bewilligt = true;

      expect(component.bescheidBewilligt).toBeTruthy();
    });

    it('should return false', () => {
      component.bescheidStateResource.resource.bewilligt = false;

      expect(component.bescheidBewilligt).toBeFalsy();
    });
  });

  describe('bescheidAbgelehnt', () => {
    it('should return false', () => {
      component.bescheidStateResource.resource.bewilligt = true;

      expect(component.bescheidAbgelehnt).toBeFalsy();
    });

    it('should return true', () => {
      component.bescheidStateResource.resource.bewilligt = false;

      expect(component.bescheidAbgelehnt).toBeTruthy();
    });
  });
});
