import {
  createDialogRefMock,
  DialogRefMock,
  existsAsHtmlElement,
  mock,
  Mock,
  triggerEvent,
  useFromMock,
} from '@alfa-client/test-utils';
import { OzgcloudDialogService } from '@alfa-client/ui';
import { VorgangWithEingangResource } from '@alfa-client/vorgang-shared';
import { EventEmitter } from '@angular/core';
import { ComponentFixture, TestBed } from '@angular/core/testing';
import { of } from 'rxjs';
import { createSuccessfullyDoneCommandStateResource } from '../../../../../../command-shared/test/command';
import { getDataTestIdOf } from '../../../../../../tech-shared/test/data-test';
import {
  createDialogResult,
  OzgcloudDialogCommandResult,
} from '../../../../../../ui/src/lib/ui/ozgcloud-dialog/ozgcloud-dialog.result';
import { createVorgangWithEingangResource } from '../../../../../../vorgang-shared/test/vorgang';
import {
  AbschliessenDialogData,
  BescheidWizardAbschliessenDialogContainerComponent,
} from '../abschliessen-dialog-container/bescheid-wizard-abschliessen-dialog-container.component';
import { BescheidWizardAbschliessenButtonComponent } from './bescheid-wizard-abschliessen-button.component';

describe('BescheidWizardAbschliessenButtonComponent', () => {
  let component: BescheidWizardAbschliessenButtonComponent;
  let fixture: ComponentFixture<BescheidWizardAbschliessenButtonComponent>;

  const abschliessenButton: string = getDataTestIdOf('bescheid-ueberspringen');

  let ozgcloudDialogService: Mock<OzgcloudDialogService>;

  beforeEach(() => {
    ozgcloudDialogService = mock(OzgcloudDialogService);
  });

  beforeEach(async () => {
    await TestBed.configureTestingModule({
      declarations: [BescheidWizardAbschliessenButtonComponent],
      providers: [
        {
          provide: OzgcloudDialogService,
          useValue: ozgcloudDialogService,
        },
      ],
    }).compileComponents();

    fixture = TestBed.createComponent(BescheidWizardAbschliessenButtonComponent);
    component = fixture.componentInstance;
    component.vorgangAbgeschlossen = useFromMock(mock(EventEmitter<void>));
    fixture.detectChanges();
  });

  it('should create', () => {
    expect(component).toBeTruthy();
  });

  describe('component', () => {
    describe('onClick', () => {
      beforeEach(() => {
        component.handleDialogClosed = jest.fn();
      });

      it('should open dialog', () => {
        const vorgangWithEingangResource: VorgangWithEingangResource = createVorgangWithEingangResource();
        component.vorgangWithEingangResource = vorgangWithEingangResource;

        component.onClick();

        expect(ozgcloudDialogService.openInCallingComponentContext).toHaveBeenCalledWith(
          BescheidWizardAbschliessenDialogContainerComponent,
          component.viewContainerRef,
          { vorgangWithEingangResource } as AbschliessenDialogData,
        );
      });

      it('should handle dialog closed', () => {
        const dialogRef: DialogRefMock<OzgcloudDialogCommandResult> = createDialogRefMock();
        ozgcloudDialogService.openInCallingComponentContext.mockReturnValue(dialogRef);

        component.onClick();

        expect(component.handleDialogClosed).toHaveBeenCalledWith(dialogRef);
      });
    });
  });

  describe('handleDialogClosed', () => {
    let dialogRef: DialogRefMock<OzgcloudDialogCommandResult>;

    beforeEach(() => {
      dialogRef = createDialogRefMock();
    });

    it('should NOT emit vorgang abgeschlossen', () => {
      dialogRef.closed = of(undefined);

      component.handleDialogClosed(dialogRef as never);

      expect(component.vorgangAbgeschlossen.emit).not.toHaveBeenCalled();
    });

    it('should emit vorgang abgeschlossen', () => {
      dialogRef.closed = of(createDialogResult(createSuccessfullyDoneCommandStateResource()));

      component.handleDialogClosed(dialogRef as never);

      expect(component.vorgangAbgeschlossen.emit).toHaveBeenCalled();
    });
  });

  describe('template', () => {
    it('should have button', () => {
      existsAsHtmlElement(fixture, abschliessenButton);
    });

    describe('output', () => {
      it('should call onClick', () => {
        component.onClick = jest.fn();

        triggerEvent({ fixture, name: 'click', elementSelector: abschliessenButton });

        expect(component.onClick).toHaveBeenCalled();
      });
    });
  });
});
