import { OzgcloudDialogService } from '@alfa-client/ui';
import { VorgangWithEingangResource } from '@alfa-client/vorgang-shared';
import { DialogRef } from '@angular/cdk/dialog';
import { Component, EventEmitter, Input, Output, ViewContainerRef } from '@angular/core';
import { filter } from 'rxjs';
import {
  OzgcloudDialogCommandResult,
  isDialogSuccessfullyCompleted,
} from '../../../../../../ui/src/lib/ui/ozgcloud-dialog/ozgcloud-dialog.result';
import {
  AbschliessenDialogData,
  BescheidWizardAbschliessenDialogContainerComponent,
} from '../abschliessen-dialog-container/bescheid-wizard-abschliessen-dialog-container.component';

@Component({
  selector: 'alfa-bescheid-wizard-abschliessen-button',
  templateUrl: './bescheid-wizard-abschliessen-button.component.html',
})
export class BescheidWizardAbschliessenButtonComponent {
  @Input() vorgangWithEingangResource: VorgangWithEingangResource;

  @Output() vorgangAbgeschlossen: EventEmitter<void> = new EventEmitter<void>();

  constructor(
    private readonly ozgclouDialogService: OzgcloudDialogService,
    readonly viewContainerRef: ViewContainerRef,
  ) {}

  public onClick(): void {
    const dialogData: AbschliessenDialogData = {
      vorgangWithEingangResource: this.vorgangWithEingangResource,
    };

    const dialogRef: DialogRef<OzgcloudDialogCommandResult> =
      this.ozgclouDialogService.openInCallingComponentContext(
        BescheidWizardAbschliessenDialogContainerComponent,
        this.viewContainerRef,
        dialogData,
      );

    this.handleDialogClosed(dialogRef);
  }

  handleDialogClosed(dialogRef: DialogRef<OzgcloudDialogCommandResult>): void {
    dialogRef.closed
      .pipe(filter(isDialogSuccessfullyCompleted))
      .subscribe(() => this.vorgangAbgeschlossen.emit());
  }
}
