import { Bescheid } from '@alfa-client/bescheid-shared';
import { existsAsHtmlElement, getElementFromFixtureByType, Mock, mock, notExistsAsHtmlElement } from '@alfa-client/test-utils';
import { ComponentFixture, TestBed } from '@angular/core/testing';
import { BescheidStatusTextComponent } from '@ods/system';
import { MockComponent } from 'ng-mocks';
import { EMPTY, of } from 'rxjs';
import { createBescheid } from '../../../../../../../bescheid-shared/src/test/bescheid';
import { getDataTestIdOf } from '../../../../../../../tech-shared/test/data-test';
import { BescheidFormService } from '../../../bescheid.formservice';
import { BescheidWizardAntragBescheidenSummaryComponent } from './bescheid-wizard-antrag-bescheiden-summary.component';

describe('BescheidWizardAntragBescheidenSummaryComponent', () => {
  let component: BescheidWizardAntragBescheidenSummaryComponent;
  let fixture: ComponentFixture<BescheidWizardAntragBescheidenSummaryComponent>;

  const bescheidStatusText: string = getDataTestIdOf('bescheid-status-text');

  let formService: Mock<BescheidFormService>;

  beforeEach(() => {
    formService = mock(BescheidFormService);
  });

  beforeEach(async () => {
    await TestBed.configureTestingModule({
      declarations: [BescheidWizardAntragBescheidenSummaryComponent, MockComponent(BescheidStatusTextComponent)],
      providers: [{ provide: BescheidFormService, useValue: formService }],
    }).compileComponents();

    fixture = TestBed.createComponent(BescheidWizardAntragBescheidenSummaryComponent);
    component = fixture.componentInstance;
    fixture.detectChanges();
  });

  it('should create', () => {
    expect(component).toBeTruthy();
  });

  describe('template', () => {
    describe('ods-bescheid-status-text', () => {
      function getElementComponent(): BescheidStatusTextComponent {
        return getElementFromFixtureByType(fixture, BescheidStatusTextComponent);
      }

      it('should subscribe to bescheid value form changes', () => {
        fixture.detectChanges();

        expect(formService.getBescheidFormValueChanges).toHaveBeenCalled();
      });

      it('should NOT show', () => {
        formService.getBescheidFormValueChanges.mockReturnValue(EMPTY);

        fixture.detectChanges();

        notExistsAsHtmlElement(fixture, bescheidStatusText);
      });

      it('should show', () => {
        formService.getBescheidFormValueChanges.mockReturnValue(of(createBescheid()));

        fixture.detectChanges();

        existsAsHtmlElement(fixture, bescheidStatusText);
      });

      describe('input', () => {
        it('should set bewilligt', () => {
          const bescheid: Bescheid = createBescheid();
          formService.getBescheidFormValueChanges.mockReturnValue(of(bescheid));

          fixture.detectChanges();

          expect(getElementComponent().bewilligt).toBe(bescheid.bewilligt);
        });

        it('should set dateText', () => {
          const bescheid: Bescheid = { ...createBescheid(), beschiedenAm: '2024-01-01' };
          formService.getBescheidFormValueChanges.mockReturnValue(of(bescheid));

          fixture.detectChanges();

          expect(getElementComponent().dateText).toBe('01.01.2024');
        });

        it('should set hasBescheidDraft', () => {
          const bescheid: Bescheid = createBescheid();
          formService.getBescheidFormValueChanges.mockReturnValue(of(bescheid));

          fixture.detectChanges();

          expect(getElementComponent().hasBescheidDraft).toBeFalsy();
        });
      });
    });
  });
});
