import { BescheidResource, BescheidService, BescheidWizardStep } from '@alfa-client/bescheid-shared';
import { HasLinkPipe, StateResource, createEmptyStateResource } from '@alfa-client/tech-shared';
import {
  Mock,
  existsAsHtmlElement,
  getElementFromFixtureByType,
  mock,
  notExistsAsHtmlElement,
  triggerEvent,
  useFromMock,
} from '@alfa-client/test-utils';
import { SpinnerComponent } from '@alfa-client/ui';
import { VorgangWithEingangLinkRel } from '@alfa-client/vorgang-shared';
import { EventEmitter } from '@angular/core';
import { ComponentFixture, TestBed } from '@angular/core/testing';
import { ReactiveFormsModule, UntypedFormBuilder } from '@angular/forms';
import { MockComponent } from 'ng-mocks';
import { createBescheidStateResource } from '../../../../../bescheid-shared/src/test/bescheid';
import { createSuccessfullyDoneCommandStateResource } from '../../../../../command-shared/test/command';
import { getDataTestIdOf } from '../../../../../tech-shared/test/data-test';
import { createVorgangWithEingangResource } from '../../../../../vorgang-shared/test/vorgang';
import { BescheidFormService } from '../bescheid.formservice';
import { BescheidWizardAntragBescheidenComponent } from './antrag-bescheiden/bescheid-wizard-antrag-bescheiden.component';
import { BescheidWizardComponent } from './bescheid-wizard.component';
import { BescheidWizardDokumenteHochladenComponent } from './dokumente-hochladen/bescheid-wizard-dokumente-hochladen.component';
import { BescheidWizardStepperComponent } from './stepper/bescheid-wizard-stepper.component';

describe('BescheidWizardComponent', () => {
  let component: BescheidWizardComponent;
  let fixture: ComponentFixture<BescheidWizardComponent>;

  const bescheidLoadingSpinner: string = getDataTestIdOf('bescheid-loading-spinner');
  const antragBescheidenStep: string = getDataTestIdOf('antrag-bescheiden-step');
  const dokumenteHochladenStep: string = getDataTestIdOf('dokumente-hochladen-step');

  let bescheidService: Mock<BescheidService>;
  let formService: BescheidFormService;

  beforeEach(() => {
    bescheidService = mock(BescheidService);
    formService = new BescheidFormService(new UntypedFormBuilder(), useFromMock(bescheidService));
  });

  beforeEach(async () => {
    await TestBed.configureTestingModule({
      declarations: [
        BescheidWizardComponent,
        MockComponent(BescheidWizardStepperComponent),
        MockComponent(BescheidWizardAntragBescheidenComponent),
        MockComponent(BescheidWizardDokumenteHochladenComponent),
        MockComponent(SpinnerComponent),
        HasLinkPipe,
      ],
      providers: [
        {
          provide: BescheidFormService,
          useValue: formService,
        },
      ],
      imports: [ReactiveFormsModule],
    }).compileComponents();

    fixture = TestBed.createComponent(BescheidWizardComponent);
    component = fixture.componentInstance;
    component.weiterClickEmitter = useFromMock(mock(EventEmitter<BescheidWizardStep>));
    component.vorgangAbgeschlossen = useFromMock(mock(EventEmitter<void>));
    fixture.detectChanges();
  });

  it('should create', () => {
    expect(component).toBeTruthy();
  });

  describe('component', () => {
    describe('set bescheidDraftStateResource', () => {
      it('should patch form values', () => {
        formService.patchValues = jest.fn();

        component.bescheidStateResource = createBescheidStateResource();

        expect(formService.patchValues).toHaveBeenCalledWith(component.bescheidStateResource.resource);
      });

      it('should NOT patch form values', () => {
        formService.patchValues = jest.fn();

        component.bescheidStateResource = createEmptyStateResource();

        expect(formService.patchValues).not.toHaveBeenCalled();
      });

      it('should set bescheid state resource', () => {
        component.bescheidStateResource = createBescheidStateResource();

        expect(component.bescheidStateResource).toEqual(component.bescheidStateResource);
      });
    });
  });

  describe('template', () => {
    beforeEach(() => {
      component.bescheidStateResource = createBescheidStateResource();
    });

    describe('ozgcloud-spinner', () => {
      it('should exists', () => {
        component.vorgangWithEingangResource = createVorgangWithEingangResource([VorgangWithEingangLinkRel.BESCHEID_DRAFT]);
        component.bescheidStateResource = createEmptyStateResource();

        fixture.detectChanges();

        existsAsHtmlElement(fixture, bescheidLoadingSpinner);
      });

      it('should NOT exists if bescheid does not exists', () => {
        component.vorgangWithEingangResource = createVorgangWithEingangResource();
        component.bescheidStateResource = createEmptyStateResource();

        fixture.detectChanges();

        notExistsAsHtmlElement(fixture, bescheidLoadingSpinner);
      });

      it('should NOT exists if bescheid initially loaded', () => {
        component.vorgangWithEingangResource = createVorgangWithEingangResource([VorgangWithEingangLinkRel.BESCHEID_DRAFT]);
        component.bescheidStateResource = createBescheidStateResource();

        fixture.detectChanges();

        notExistsAsHtmlElement(fixture, bescheidLoadingSpinner);
      });

      describe('input', () => {
        it('should set stateResource', () => {
          component.vorgangWithEingangResource = createVorgangWithEingangResource([VorgangWithEingangLinkRel.BESCHEID_DRAFT]);
          component.bescheidStateResource = createEmptyStateResource();

          fixture.detectChanges();

          expect(getElementFromFixtureByType(fixture, SpinnerComponent).stateResource).toEqual(component.bescheidStateResource);
        });
      });
    });

    describe('alfa-bescheid-wizard-antrag-bescheiden', () => {
      function getElementComponent(): BescheidWizardAntragBescheidenComponent {
        return getElementFromFixtureByType(fixture, BescheidWizardAntragBescheidenComponent);
      }

      it('should show', () => {
        givenActiveStep(1);
        givenLoadedBescheidStateResrouce();

        existsAsHtmlElement(fixture, antragBescheidenStep);
      });

      it.each([2, 3])('should NOT show in step %d', (step: number) => {
        givenActiveStep(step);
        givenLoadedBescheidStateResrouce();

        notExistsAsHtmlElement(fixture, antragBescheidenStep);
      });

      describe('input', () => {
        it('should set vorgangWithEingangResource', () => {
          givenActiveStep(1);
          givenLoadedBescheidStateResrouce();
          component.vorgangWithEingangResource = createVorgangWithEingangResource();

          fixture.detectChanges();

          expect(getElementComponent().vorgangWithEingangResource).toBe(component.vorgangWithEingangResource);
        });

        it('should set bescheidResource', () => {
          givenActiveStep(1);
          givenLoadedBescheidStateResrouce();

          fixture.detectChanges();

          expect(getElementComponent().bescheidResource).toEqual(component.bescheidStateResource.resource);
        });

        it('should set submitStateResource', () => {
          givenActiveStep(1);
          givenLoadedBescheidStateResrouce();
          component.submitStateResource = createSuccessfullyDoneCommandStateResource();

          fixture.detectChanges();

          expect(getElementComponent().submitStateResource).toEqual(component.submitStateResource);
        });
      });

      describe('output', () => {
        beforeEach(() => {
          givenActiveStep(1);
          givenLoadedBescheidStateResrouce();
        });

        it('should emit weiterClickEmitter', () => {
          triggerEvent({
            fixture,
            name: 'weiterClickEmitter',
            elementSelector: antragBescheidenStep,
            data: BescheidWizardStep.DokumenteHochladen,
          });

          expect(component.weiterClickEmitter.emit).toHaveBeenCalledWith(BescheidWizardStep.DokumenteHochladen);
        });

        it('should emit vorgangAbgeschlossen', () => {
          triggerEvent({
            fixture,
            name: 'vorgangAbgeschlossen',
            elementSelector: antragBescheidenStep,
          });

          expect(component.vorgangAbgeschlossen.emit).toHaveBeenCalled();
        });
      });
    });

    describe('alfa-bescheid-wizard-dokumente-hochladen', () => {
      it('should show', () => {
        givenActiveStep(2);
        givenLoadedBescheidStateResrouce();

        existsAsHtmlElement(fixture, dokumenteHochladenStep);
      });

      it.each([1, 3])('should NOT show in step %d', (step: number) => {
        givenActiveStep(step);
        givenLoadedBescheidStateResrouce();

        notExistsAsHtmlElement(fixture, dokumenteHochladenStep);
      });
    });
  });

  function givenActiveStep(step: number): void {
    component.activeStep = step;
    fixture.detectChanges();
  }

  function givenLoadedBescheidStateResrouce(): StateResource<BescheidResource> {
    const resource: StateResource<BescheidResource> = createBescheidStateResource();
    component.bescheidStateResource = resource;
    fixture.detectChanges();
    return resource;
  }
});
