import { BescheidResource, BescheidService } from '@alfa-client/bescheid-shared';
import { tapOnCommandSuccessfullyDone } from '@alfa-client/command-shared';
import { StateResource, isNotNil } from '@alfa-client/tech-shared';
import { DIALOG_DATA, DialogRef } from '@angular/cdk/dialog';
import { Component, Inject } from '@angular/core';
import { Resource } from '@ngxp/rest';
import { Observable } from 'rxjs';
import { BescheidFormService } from '../../bescheid.formservice';

export interface CancelWizardDialogData {
  bescheidResource: BescheidResource;
}

export interface CancelWizardDialogResult {
  closeWizard: boolean;
}

@Component({
  templateUrl: './bescheid-wizard-cancel-dialog-container.component.html',
})
export class BescheidWizardCancelDialogContainerComponent {
  public saveStateResource$: Observable<StateResource<Resource>>;
  public deleteStateResource$: Observable<StateResource<Resource>>;

  bescheidResource: BescheidResource;

  constructor(
    @Inject(DIALOG_DATA) readonly dialogData: CancelWizardDialogData,
    public dialogRef: DialogRef<CancelWizardDialogResult>,
    private readonly bescheidService: BescheidService,
    private readonly formService: BescheidFormService,
  ) {
    this.bescheidResource = dialogData.bescheidResource;
  }

  public save(): void {
    this.saveStateResource$ = this.formService.submit().pipe(tapOnCommandSuccessfullyDone(() => this.closeDialogWithWizard()));
  }

  public cancel(): void {
    if (isNotNil(this.bescheidResource)) {
      this.deleteStateResource$ = this.bescheidService
        .bescheidVerwerfen()
        .pipe(tapOnCommandSuccessfullyDone(() => this.closeDialogWithWizard()));
    } else {
      this.closeDialogWithWizard();
    }
  }

  closeDialogWithWizard(): void {
    this.dialogRef.close({ closeWizard: true });
  }
}
