import { BescheidWizardStep } from '@alfa-client/bescheid-shared';
import { getElementComponentFromFixtureByCss, triggerEvent } from '@alfa-client/test-utils';
import { ComponentFixture, TestBed } from '@angular/core/testing';
import { MockComponent } from 'ng-mocks';
import { getDataTestIdOf } from '../../../../../../tech-shared/test/data-test';
import { BescheidWizardStepperComponent } from './bescheid-wizard-stepper.component';
import { BescheidWizardStepComponent } from './step/bescheid-wizard-step.component';

describe('BescheidWizardStepperComponent', () => {
  let component: BescheidWizardStepperComponent;
  let fixture: ComponentFixture<BescheidWizardStepperComponent>;

  const antragBescheidenWizardStep: string = getDataTestIdOf('wizard-step-' + BescheidWizardStep.AntragBescheiden);
  const dokumenteHochladenWizardStep: string = getDataTestIdOf('wizard-step-' + BescheidWizardStep.DokumenteHochladen);
  const bescheidVersendenWizardStep: string = getDataTestIdOf('wizard-step-' + BescheidWizardStep.BescheidVersenden);

  beforeEach(async () => {
    await TestBed.configureTestingModule({
      declarations: [BescheidWizardStepperComponent, MockComponent(BescheidWizardStepComponent)],
    }).compileComponents();

    fixture = TestBed.createComponent(BescheidWizardStepperComponent);
    component = fixture.componentInstance;
    fixture.detectChanges();
  });

  it('should create', () => {
    expect(component).toBeTruthy();
  });

  it('should have number of steps', () => {
    expect(component.steps).toEqual([
      BescheidWizardStep.AntragBescheiden,
      BescheidWizardStep.DokumenteHochladen,
      BescheidWizardStep.BescheidVersenden,
    ]);
  });

  describe('template', () => {
    describe('alfa-bescheid-wizard-step', () => {
      describe('input', () => {
        it('should set activeStep for antrag bescheiden step', () => {
          component.activeStep = 2;

          fixture.detectChanges();

          const step: BescheidWizardStepComponent = getElementComponentFromFixtureByCss<BescheidWizardStepComponent>(
            fixture,
            antragBescheidenWizardStep,
          );
          expect(step.activeStep).toEqual(component.activeStep);
        });

        it('should set activeStep for dokumente hochladen step', () => {
          component.activeStep = 2;

          fixture.detectChanges();

          const step: BescheidWizardStepComponent = getElementComponentFromFixtureByCss<BescheidWizardStepComponent>(
            fixture,
            dokumenteHochladenWizardStep,
          );
          expect(step.activeStep).toEqual(component.activeStep);
        });

        it('should set activeStep for bescheid versenden step', () => {
          component.activeStep = 2;

          fixture.detectChanges();

          const step: BescheidWizardStepComponent = getElementComponentFromFixtureByCss<BescheidWizardStepComponent>(
            fixture,
            bescheidVersendenWizardStep,
          );
          expect(step.activeStep).toEqual(component.activeStep);
        });

        it('should set step for antrag bescheiden', () => {
          fixture.detectChanges();

          const step: BescheidWizardStepComponent = getElementComponentFromFixtureByCss<BescheidWizardStepComponent>(
            fixture,
            antragBescheidenWizardStep,
          );
          expect(step.step).toEqual(BescheidWizardStep.AntragBescheiden);
        });

        it('should set step for dokumente hochladen', () => {
          fixture.detectChanges();

          const step: BescheidWizardStepComponent = getElementComponentFromFixtureByCss<BescheidWizardStepComponent>(
            fixture,
            dokumenteHochladenWizardStep,
          );
          expect(step.step).toEqual(BescheidWizardStep.DokumenteHochladen);
        });

        it('should set step for bescheid versenden', () => {
          fixture.detectChanges();

          const step: BescheidWizardStepComponent = getElementComponentFromFixtureByCss<BescheidWizardStepComponent>(
            fixture,
            bescheidVersendenWizardStep,
          );
          expect(step.step).toEqual(BescheidWizardStep.BescheidVersenden);
        });
      });

      describe('output', () => {
        it('should emit stepChange for antrag bescheiden step', () => {
          component.stepChange.emit = jest.fn();

          triggerEvent({
            fixture,
            name: 'stepChange',
            elementSelector: antragBescheidenWizardStep,
            data: BescheidWizardStep.DokumenteHochladen,
          });

          expect(component.stepChange.emit).toHaveBeenCalledWith(BescheidWizardStep.DokumenteHochladen);
        });

        it('should emit stepChange for dokumente hochladen step', () => {
          component.stepChange.emit = jest.fn();

          triggerEvent({
            fixture,
            name: 'stepChange',
            elementSelector: dokumenteHochladenWizardStep,
            data: BescheidWizardStep.DokumenteHochladen,
          });

          expect(component.stepChange.emit).toHaveBeenCalledWith(BescheidWizardStep.DokumenteHochladen);
        });

        it('should emit stepChange for bescheid versenden step', () => {
          component.stepChange.emit = jest.fn();

          triggerEvent({
            fixture,
            name: 'stepChange',
            elementSelector: bescheidVersendenWizardStep,
            data: BescheidWizardStep.DokumenteHochladen,
          });

          expect(component.stepChange.emit).toHaveBeenCalledWith(BescheidWizardStep.DokumenteHochladen);
        });
      });
    });
  });
});
