import { BescheidWizardStep } from '@alfa-client/bescheid-shared';
import { ComponentFixture, TestBed } from '@angular/core/testing';
import { BescheidWizardStepComponent } from './bescheid-wizard-step.component';

describe('BescheidWizardStepComponent', () => {
  let component: BescheidWizardStepComponent;
  let fixture: ComponentFixture<BescheidWizardStepComponent>;

  beforeEach(async () => {
    await TestBed.configureTestingModule({
      declarations: [BescheidWizardStepComponent],
    }).compileComponents();

    fixture = TestBed.createComponent(BescheidWizardStepComponent);
    component = fixture.componentInstance;
    fixture.detectChanges();
  });

  it('should create', () => {
    expect(component).toBeTruthy();
  });

  describe('isActive', () => {
    beforeEach(() => {
      component.activeStep = BescheidWizardStep.DokumenteHochladen;
    });

    it('return true if step equals activeStep', () => {
      component.step = BescheidWizardStep.DokumenteHochladen;

      const isActive: boolean = component.isActive();

      expect(isActive).toBeTruthy();
    });

    it('return false if step not equals activeStep', () => {
      component.step = BescheidWizardStep.AntragBescheiden;

      const isActive: boolean = component.isActive();

      expect(isActive).toBeFalsy();
    });
  });

  describe('isPrevious', () => {
    beforeEach(() => {
      component.activeStep = BescheidWizardStep.DokumenteHochladen;
    });

    it('return true if step is less than activeStep', () => {
      component.step = BescheidWizardStep.AntragBescheiden;

      const isPrevious: boolean = component.isPrevious();

      expect(isPrevious).toBeTruthy();
    });

    it('return false if step equals activeStep', () => {
      component.step = BescheidWizardStep.DokumenteHochladen;

      const isPrevious: boolean = component.isPrevious();

      expect(isPrevious).toBeFalsy();
    });

    it('return false if step is greater than activeStep', () => {
      component.step = BescheidWizardStep.BescheidVersenden;

      const isPrevious: boolean = component.isPrevious();

      expect(isPrevious).toBeFalsy();
    });
  });

  describe('isNext', () => {
    beforeEach(() => {
      component.activeStep = BescheidWizardStep.DokumenteHochladen;
    });

    it('return false if step is less than activeStep', () => {
      component.step = BescheidWizardStep.AntragBescheiden;

      const isNext: boolean = component.isNext();

      expect(isNext).toBeFalsy();
    });

    it('return false if step equals activeStep', () => {
      component.step = BescheidWizardStep.DokumenteHochladen;

      const isNext: boolean = component.isNext();

      expect(isNext).toBeFalsy();
    });

    it('return true if step is greater than activeStep', () => {
      component.step = BescheidWizardStep.BescheidVersenden;

      const isNext: boolean = component.isNext();

      expect(isNext).toBeTruthy();
    });
  });

  describe('clickHandler', () => {
    beforeEach(() => {
      component.stepChange.emit = jest.fn();
    });

    it('should emit step', () => {
      component.isPrevious = jest.fn().mockReturnValue(true);

      component.clickHandler(BescheidWizardStep.AntragBescheiden);

      expect(component.stepChange.emit).toHaveBeenCalledWith(BescheidWizardStep.AntragBescheiden);
    });

    it('should not emit step', () => {
      component.isPrevious = jest.fn().mockReturnValue(false);

      component.clickHandler(BescheidWizardStep.AntragBescheiden);

      expect(component.stepChange.emit).not.toHaveBeenCalled();
    });
  });
});
