/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { ComponentFixture, TestBed } from '@angular/core/testing';
import { ApiRootService } from '@alfa-client/api-root-shared';
import { BinaryFileResource, BinaryFileService } from '@alfa-client/binary-file-shared';
import { ConvertForDataTestPipe } from '@alfa-client/tech-shared';
import { mock } from '@alfa-client/test-utils';
import { createBinaryFileResource } from 'libs/binary-file-shared/test/binary-file';
import { MockComponent } from 'ng-mocks';
import { of } from 'rxjs';
import { BinaryFileContainerComponent } from './binary-file-container.component';
import { BinaryFileComponent } from './binary-file/binary-file.component';

describe('BinaryFileContainerComponent', () => {
  let component: BinaryFileContainerComponent;
  let fixture: ComponentFixture<BinaryFileContainerComponent>;

  const service = mock(BinaryFileService);
  const apiRootService = mock(ApiRootService);

  beforeEach(async () => {
    await TestBed.configureTestingModule({
      declarations: [
        BinaryFileContainerComponent,
        ConvertForDataTestPipe,
        MockComponent(BinaryFileComponent),
      ],
      providers: [
        {
          provide: BinaryFileService,
          useValue: service,
        },
        {
          provide: ApiRootService,
          useValue: apiRootService,
        },
      ],
    }).compileComponents();
  });

  beforeEach(() => {
    fixture = TestBed.createComponent(BinaryFileContainerComponent);
    component = fixture.componentInstance;
    component.file = createBinaryFileResource();

    fixture.detectChanges();
  });

  it('should create', () => {
    expect(component).toBeTruthy();
  });

  describe('start download', () => {
    const file: BinaryFileResource = createBinaryFileResource();
    const fileNameDownloadPrefix: string = 'FileNamePrefixDummy';

    it('should call file service downloadFile', () => {
      component.downloadFileNamePrefix = fileNameDownloadPrefix;

      component.startDownload(file);

      expect(service.downloadFile).toHaveBeenCalledWith(file, fileNameDownloadPrefix);
    });
  });

  describe('get download token', () => {
    it('should call ApiRootService.getDownloadToken()', () => {
      apiRootService.getDownloadToken.mockReturnValue(of(''));
      component.getDownloadToken();

      expect(apiRootService.getDownloadToken).toHaveBeenCalled();
    });
  });
});
