import { ApiDownloadToken } from '@alfa-client/api-root-shared';
import { BinaryFileLinkRel, BinaryFileResource } from '@alfa-client/binary-file-shared';
import { SpinnerComponent } from '@alfa-client/ui';
import { ComponentFixture, TestBed, fakeAsync, tick } from '@angular/core/testing';
import { faker } from '@faker-js/faker';
import { AttachmentComponent, CloseIconComponent } from '@ods/system';
import { createBinaryFileResource } from 'libs/binary-file-shared/test/binary-file';
import { FileSizePlainPipe } from 'libs/tech-shared/src/lib/pipe/file-size-plain.pipe';
import { MockComponent, MockPipe } from 'ng-mocks';
import { BinaryFile2Component } from './binary-file2.component';

describe('BinaryFile2Component', () => {
  let component: BinaryFile2Component;
  let fixture: ComponentFixture<BinaryFile2Component>;

  const file: BinaryFileResource = createBinaryFileResource([BinaryFileLinkRel.DOWNLOAD]);

  beforeEach(async () => {
    await TestBed.configureTestingModule({
      declarations: [
        BinaryFile2Component,
        MockComponent(AttachmentComponent),
        MockComponent(SpinnerComponent),
        MockComponent(CloseIconComponent),
        MockPipe(FileSizePlainPipe),
      ],
    });
  });

  beforeEach(() => {
    fixture = TestBed.createComponent(BinaryFile2Component);
    component = fixture.componentInstance;
    component.file = file;
    fixture.detectChanges();
  });

  it('should create', () => {
    expect(component).toBeTruthy();
  });

  describe('click on download button', () => {
    it('should emit download if link exists', () => {
      jest.spyOn(component.startDownload, 'emit');
      component.file = createBinaryFileResource([BinaryFileLinkRel.DOWNLOAD]);

      component.downloadFile();

      expect(component.startDownload.emit).toHaveBeenCalledWith(component.file);
    });
  });

  describe('click and hold download button', () => {
    it('should call getNewDownloadToken', () => {
      jest.spyOn(component.getDownloadToken, 'emit');
      component.file = createBinaryFileResource();

      component.getNewDownloadToken();
      //TODO: Test mit onmousedown
      //const button: HTMLElement = getElementFromFixture(fixture, buttonSelector);
      //const mousedown = new MouseEvent('mousedown');
      //button.dispatchEvent(mousedown);

      expect(component.getDownloadToken.emit).toHaveBeenCalled();
    });
  });

  describe('click on delete button', () => {
    it('should emit delete', () => {
      jest.spyOn(component.startDelete, 'emit');
      component.file = createBinaryFileResource();

      component.deleteFile();

      expect(component.startDelete.emit).toHaveBeenCalledWith(component.file);
    });
  });

  describe('create download url', () => {
    it('should return a DownloadURL', () => {
      component.downloadToken = { token: faker.string.alpha({ length: 30 }) } as ApiDownloadToken;
      const pattern = `^[^:]+:[^:]+:http.+\\?token=${component.downloadToken.token}$`;
      const regex = new RegExp(pattern);

      const downloadUrl = component.createDownloadUrl();

      expect(downloadUrl).toMatch(regex);
    });
  });

  describe('handle spinner for drag end', () => {
    it('should set state resource loading', () => {
      component.handleSpinnerForDragEnd();

      expect(component.stateResource.loading).toBeTruthy();
    });

    it('should unset state resource loading', fakeAsync(() => {
      component.handleSpinnerForDragEnd();

      tick(3000);

      expect(component.stateResource.loading).toBeFalsy();
    }));
  });
});
