import { BinaryFileService } from '@alfa-client/binary-file-shared';
import { Mock, dispatchEventFromFixture, mock } from '@alfa-client/test-utils';
import { ComponentFixture, TestBed } from '@angular/core/testing';
import { faker } from '@faker-js/faker';
import { DownloadButtonComponent } from '@ods/component';
import { getDataTestClassOf } from 'libs/tech-shared/test/data-test';
import { MockComponent } from 'ng-mocks';
import { DownloadArchiveFileButtonContainerComponent } from './download-archive-file-button-container.component';

describe('DownloadArchiveFileButtonContainerComponent', () => {
  let component: DownloadArchiveFileButtonContainerComponent;
  let fixture: ComponentFixture<DownloadArchiveFileButtonContainerComponent>;

  const downloadArchiveButton: string = getDataTestClassOf('download-archive');

  const binaryFileService: Mock<BinaryFileService> = mock(BinaryFileService);

  beforeEach(async () => {
    await TestBed.configureTestingModule({
      declarations: [DownloadArchiveFileButtonContainerComponent],
      imports: [MockComponent(DownloadButtonComponent)],
      providers: [
        {
          provide: BinaryFileService,
          useValue: binaryFileService,
        },
      ],
    }).compileComponents();

    fixture = TestBed.createComponent(DownloadArchiveFileButtonContainerComponent);
    component = fixture.componentInstance;
    fixture.detectChanges();
  });

  it('should create', () => {
    expect(component).toBeTruthy();
  });

  describe('download attachments', () => {
    const downloadUri: string = faker.internet.url();

    it('should call service to download archive', () => {
      component.downloadUri = downloadUri;
      dispatchEventFromFixture(fixture, downloadArchiveButton, 'click');

      expect(binaryFileService.downloadArchive).toHaveBeenCalledWith(downloadUri);
    });
  });
});
