import { CommandOrder, CommandResource, CommandService } from '@alfa-client/command-shared';
import { StateResource } from '@alfa-client/tech-shared';
import { Injectable } from '@angular/core';
import { BehaviorSubject, Observable } from 'rxjs';
import { CollaborationListResourceService } from './collaboration-list-resource.service';
import { CollaborationListLinkRel } from './collaboration.linkrel';
import { Collaboration, CollaborationListResource } from './collaboration.model';

@Injectable()
export class CollaborationService {
  showExterneFachstelleForm$: BehaviorSubject<boolean> = new BehaviorSubject(false);
  showExterneFachstelleAnfrageButton$: BehaviorSubject<boolean> = new BehaviorSubject(true);

  showOrganisationsEinheitForm$: BehaviorSubject<boolean> = new BehaviorSubject(false);
  showOrganisationsEinheitAnfrageButton$: BehaviorSubject<boolean> = new BehaviorSubject(true);

  collaborationLevel$: BehaviorSubject<number> = new BehaviorSubject(null);

  readonly ORGANISATIONS_EINHEIT_COLLABORATION_LEVEL: number = 1;
  readonly EXTERNE_FACHSTELLE_COLLABORATION_LEVEL: number = 4;

  constructor(
    private listService: CollaborationListResourceService,
    private commandService: CommandService,
  ) {}

  public getList(): Observable<StateResource<CollaborationListResource>> {
    return this.listService.getList();
  }

  public isOrganisationsEinheitFormVisible(): Observable<boolean> {
    return this.showOrganisationsEinheitForm$.asObservable();
  }

  public showOrganisationsEinheitForm(): void {
    this.showOrganisationsEinheitForm$.next(true);
    this.showExterneFachstelleAnfrageButton$.next(false);
    this.collaborationLevel$.next(this.ORGANISATIONS_EINHEIT_COLLABORATION_LEVEL);
  }

  public hideOrganisationseinheitForm(): void {
    this.showOrganisationsEinheitForm$.next(false);
    this.showExterneFachstelleAnfrageButton$.next(true);
    this.collaborationLevel$.next(null);
  }

  public showOrganisationsEinheitAnfrageButton(): Observable<boolean> {
    return this.showOrganisationsEinheitAnfrageButton$.asObservable();
  }

  public isExterneFachstelleFormVisible(): Observable<boolean> {
    return this.showExterneFachstelleForm$.asObservable();
  }

  public showExterneFachstelleForm(): void {
    this.showExterneFachstelleForm$.next(true);
    this.showOrganisationsEinheitAnfrageButton$.next(false);
    this.collaborationLevel$.next(this.EXTERNE_FACHSTELLE_COLLABORATION_LEVEL);
  }

  public hideExterneFachstelleForm(): void {
    this.showExterneFachstelleForm$.next(false);
    this.showOrganisationsEinheitAnfrageButton$.next(true);
    this.collaborationLevel$.next(null);
  }

  public showExterneFachstelleAnfrageButton(): Observable<boolean> {
    return this.showExterneFachstelleAnfrageButton$.asObservable();
  }

  public create(
    listResource: CollaborationListResource,
    collaborationRequest: Collaboration,
  ): Observable<StateResource<CommandResource>> {
    return this.commandService.createCommandByProps({
      linkRel: CollaborationListLinkRel.CREATE_COLLABORATION_REQUEST,
      resource: listResource,
      command: {
        order: CommandOrder.CREATE_COLLABORATION_REQUEST,
        body: { ...collaborationRequest, collaborationLevel: this.collaborationLevel$.value },
      },
      snackBarMessage: 'Die Zuarbeit wurde angefragt.',
    });
  }
}
