import { CommandResource } from '@alfa-client/command-shared';
import { StateResource } from '@alfa-client/tech-shared';
import { CommonModule } from '@angular/common';
import { Component, EventEmitter, Input, Output } from '@angular/core';
import { ButtonComponent, SaveIconComponent, iconVariants } from '@ods/system';
import { VariantProps } from 'class-variance-authority';

type IconVariants = VariantProps<typeof iconVariants>;

@Component({
  selector: 'ods-download-button',
  standalone: true,
  imports: [CommonModule, ButtonComponent, SaveIconComponent],
  template: `<ods-button
    [dataTestId]="dataTestId"
    variant="icon"
    size="fit"
    spinnerSize="small"
    [isLoading]="isLoading"
    (click)="clickEmitter.emit()"
  >
    <ods-save-icon icon [size]="size" class="fill-text"></ods-save-icon>
  </ods-button>`,
})
export class DownloadButtonComponent {
  @Input() dataTestId: string = '';
  @Input() size: IconVariants['size'] = 'small';
  @Input() set stateResource(resource: StateResource<CommandResource>) {
    this.isLoading = resource.loading || resource.reload;
  }

  @Output() public clickEmitter: EventEmitter<MouseEvent> = new EventEmitter<MouseEvent>();

  isLoading: boolean = false;
}
