import { existsAsHtmlElement, getElementFromFixture, notExistsAsHtmlElement } from '@alfa-client/test-utils';
import { ComponentFixture, TestBed } from '@angular/core/testing';
import { getDataTestIdOf } from 'libs/tech-shared/test/data-test';
import { ButtonToggleComponent } from './button-toggle.component';

describe('ButtonToggleComponent', () => {
  let component: ButtonToggleComponent;
  let fixture: ComponentFixture<ButtonToggleComponent>;
  const checkIconSelector = getDataTestIdOf('toggle-check-icon');
  const buttonToggleSelector = getDataTestIdOf('button-toggle');

  beforeEach(async () => {
    await TestBed.configureTestingModule({
      imports: [ButtonToggleComponent],
    }).compileComponents();

    fixture = TestBed.createComponent(ButtonToggleComponent);
    component = fixture.componentInstance;
    fixture.detectChanges();
  });

  it('should create', () => {
    expect(component).toBeTruthy();
  });

  describe('component', () => {
    describe('isFocused', () => {
      it('should be set to true on focus', () => {
        const buttonToggle: HTMLButtonElement = getElementFromFixture(fixture, buttonToggleSelector);

        buttonToggle.focus();
        fixture.detectChanges();

        expect(component.isFocused).toBe(true);
      });

      it('should be set to false on blue', () => {
        component.isFocused = true;
        const buttonToggle: HTMLButtonElement = getElementFromFixture(fixture, buttonToggleSelector);

        buttonToggle.dispatchEvent(new Event('blur'));
        fixture.detectChanges();

        expect(component.isFocused).toBe(false);
      });
    });

    describe('isChecked', () => {
      it('should call isSelected', () => {
        component.isSelected = jest.fn();
        component.value = 'test';

        component.isChecked;

        expect(component.isSelected).toHaveBeenCalledWith('test');
      });

      it('should return result of isSelected', () => {
        component.isSelected = jest.fn().mockReturnValue(true);

        const result: boolean = component.isChecked;

        expect(result).toBe(true);
      });
    });
  });

  describe('template', () => {
    describe('check icon', () => {
      it('should show', () => {
        component.isSelected = jest.fn().mockReturnValue(true);

        fixture.detectChanges();

        existsAsHtmlElement(fixture, checkIconSelector);
      });

      it('should hide', () => {
        component.isSelected = jest.fn().mockReturnValue(false);

        fixture.detectChanges();

        notExistsAsHtmlElement(fixture, checkIconSelector);
      });
    });
  });
});
