/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { ApiErrorAction, createStateResource } from '@alfa-client/tech-shared';
import { Action } from '@ngrx/store';
import { HistorieListProps } from './historie.actions';
import { HistorieListResource } from './historie.model';
import { HistorieState, initialState, reducer } from './historie.reducer';

import * as HistorieActions from './historie.actions';

describe('Historie Reducer', () => {
  describe('unknown action', () => {
    it('should return the previous state', () => {
      const action = {} as Action;

      const result = reducer(initialState, action);

      expect(result).toBe(initialState);
    });
  });

  describe('on loadHistorie action', () => {
    it('should set loading to true', () => {
      const action = HistorieActions.loadHistorie();

      const result: HistorieState = reducer(initialState, action);

      expect(result.historieList.loading).toBeTruthy();
    });
  });

  describe('on loadHistorieSuccess action', () => {
    it('should set loaded resource into state', () => {
      const historieList: HistorieListResource = <HistorieListResource>{};
      const action = HistorieActions.loadHistorieSuccess(<HistorieListProps>{ historieList });

      const result: HistorieState = reducer(initialState, action);

      expect(result.historieList).toEqual(createStateResource(historieList));
    });
  });

  describe('on loadHistorieFailed action', () => {
    it('should set error into state', () => {
      const apiError = <Error>{}; //TODO durch richtigen Error ersetzt
      const action = HistorieActions.loadHistorieFailed(<ApiErrorAction>(<unknown>{ apiError }));

      const result: HistorieState = reducer(initialState, action);

      expect(result.historieList.error).toBe(apiError);
    });
  });

  describe('on reloadHistorie action', () => {
    it('should set reload on stateresource to true', () => {
      const action = HistorieActions.reloadHistorie();

      const result: HistorieState = reducer(initialState, action);

      expect(result.historieList.reload).toBeTruthy();
    });
  });
});
