/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { DebugElement, Type } from '@angular/core';
import { ComponentFixture } from '@angular/core/testing';
import { By } from '@angular/platform-browser';
import { EventData } from './model';

export function getElementFromFixtureByType<T>(fixture: ComponentFixture<any>, component: Type<T>): T {
  return getDebugElementFromFixtureByType(fixture, component).componentInstance as T;
}

function getDebugElementFromFixtureByType<T>(fixture: ComponentFixture<any>, component: Type<T>): DebugElement {
  return fixture.debugElement.query(By.directive(component));
}

export function getElementsFromFixtureByType<T>(fixture: ComponentFixture<any>, component: Type<T>): T[] {
  return getDebugElementsFromFixtureByType(fixture, component).map((element: DebugElement) => element.componentInstance as T);
}

function getDebugElementsFromFixtureByType<T>(fixture: ComponentFixture<any>, component: Type<T>): DebugElement[] {
  return fixture.debugElement.queryAll(By.directive(component));
}

export function getElementFromFixture(fixture: ComponentFixture<any>, htmlElement: string): any {
  return fixture.nativeElement.querySelector(htmlElement);
}

export function getElementsFromFixture(fixture: ComponentFixture<any>, htmlElement: string): any {
  return fixture.nativeElement.querySelectorAll(htmlElement);
}

export function dispatchEventFromFixture<T>(fixture: ComponentFixture<T>, elementSelector: string, event: string): void {
  const element: DebugElement = getDebugElementFromFixtureByCss(fixture, elementSelector);
  element.nativeElement.dispatchEvent(new Event(event));
}

export function triggerEvent<T>(eventData: EventData<T>) {
  const element: DebugElement = getDebugElementFromFixtureByCss(eventData.fixture, eventData.elementSelector);
  element.triggerEventHandler(eventData.name, eventData.data);
}

export function getDebugElementFromFixtureByCss(fixture: ComponentFixture<any>, query: string): DebugElement {
  return fixture.debugElement.query(By.css(query));
}

export function getElementComponentFromFixtureByCss<T>(fixture: ComponentFixture<any>, query: string): T {
  return fixture.debugElement.query(By.css(query)).componentInstance as T;
}

export function getElementFromDomRoot(fixture: ComponentFixture<any>, htmlElement: string): any {
  return fixture.nativeElement.parentNode.querySelector(htmlElement);
}

function getDebugElementFromFixtureByDirective(fixture: ComponentFixture<any>, query: Type<any>): DebugElement {
  return fixture.debugElement.query(By.directive(query));
}

function getAllDebugElementsFromFixtureByDirective(fixture: ComponentFixture<any>, query: Type<any>): DebugElement[] {
  return fixture.debugElement.queryAll(By.directive(query));
}

export function getMockComponent<T>(fixture: ComponentFixture<unknown>, component: Type<T>): T {
  return <T>getDebugElementFromFixtureByDirective(fixture, component).componentInstance;
}

export function getMockComponents<T>(fixture: ComponentFixture<unknown>, component: Type<T>): T[] {
  return getAllDebugElementsFromFixtureByDirective(fixture, component).map((debugElement) => <T>debugElement.componentInstance);
}
