import {
  BescheidLinkRel,
  BescheidResource,
  BescheidSendBy,
  BescheidService,
} from '@alfa-client/bescheid-shared';
import { HasLinkPipe, createStateResource } from '@alfa-client/tech-shared';
import { Mock, getElementFromFixture, mock, useFromMock } from '@alfa-client/test-utils';
import { ComponentFixture, TestBed } from '@angular/core/testing';
import { ReactiveFormsModule, UntypedFormBuilder } from '@angular/forms';
import { MatIconTestingModule } from '@angular/material/icon/testing';
import { RadioButtonCardComponent, SaveIconComponent, SendIconComponent } from '@ods/system';
import { MockComponent } from 'ng-mocks';
import { EMPTY, of } from 'rxjs';
import { createBescheidResource } from '../../../../../../../../bescheid-shared/src/test/bescheid';
import { getDataTestIdOf } from '../../../../../../../../tech-shared/test/data-test';
import { BescheidenFormService } from '../../../bescheiden.formservice';
import { VorgangDetailBescheidenBescheidVersendenComponent } from './vorgang-detail-bescheiden-bescheid-versenden.component';

describe('VorgangDetailBescheidenBescheidVersendenComponent', () => {
  let component: VorgangDetailBescheidenBescheidVersendenComponent;
  let fixture: ComponentFixture<VorgangDetailBescheidenBescheidVersendenComponent>;

  let bescheidService: Mock<BescheidService>;
  let formService: BescheidenFormService;

  const sendToAntragstellerButtonTestId: string = getDataTestIdOf('send-to-antragsteller-button');
  const saveAndSendButtonTestId: string = getDataTestIdOf('save-button');

  beforeEach(async () => {
    bescheidService = mock(BescheidService);
    formService = new BescheidenFormService(new UntypedFormBuilder(), useFromMock(bescheidService));

    await TestBed.configureTestingModule({
      imports: [ReactiveFormsModule, MatIconTestingModule],
      declarations: [
        VorgangDetailBescheidenBescheidVersendenComponent,
        HasLinkPipe,
        MockComponent(RadioButtonCardComponent),
        MockComponent(SendIconComponent),
        MockComponent(SaveIconComponent),
      ],
      providers: [
        {
          provide: BescheidenFormService,
          useValue: formService,
        },
        {
          provide: BescheidService,
          useValue: bescheidService,
        },
      ],
    }).compileComponents();

    fixture = TestBed.createComponent(VorgangDetailBescheidenBescheidVersendenComponent);
    component = fixture.componentInstance;
    bescheidService.getBescheidDraft.mockReturnValue(EMPTY);
    fixture.detectChanges();
  });

  it('should create', () => {
    expect(component).toBeTruthy();
  });

  describe('render', () => {
    it('should render send button', () => {
      bescheidService.getBescheidDraft.mockReturnValue(
        of(createStateResource(createBescheidResource([BescheidLinkRel.BESCHEIDEN_UND_SENDEN]))),
      );

      component.ngOnInit();
      fixture.detectChanges();

      const element = getElementFromFixture(fixture, sendToAntragstellerButtonTestId);
      expect(element).toBeInstanceOf(HTMLElement);
    });

    it('should not render send button', () => {
      bescheidService.getBescheidDraft.mockReturnValue(
        of(createStateResource(createBescheidResource())),
      );

      component.ngOnInit();
      fixture.detectChanges();

      const element = getElementFromFixture(fixture, sendToAntragstellerButtonTestId);
      expect(element).not.toBeInstanceOf(HTMLElement);
    });

    it('should render save button', () => {
      bescheidService.getBescheidDraft.mockReturnValue(
        of(
          createStateResource(
            createBescheidResource([
              BescheidLinkRel.BESCHEIDEN,
              BescheidLinkRel.BESCHEIDEN_UND_SENDEN,
            ]),
          ),
        ),
      );

      component.ngOnInit();
      fixture.detectChanges();

      const element = getElementFromFixture(fixture, saveAndSendButtonTestId);
      expect(element).toBeInstanceOf(HTMLElement);
    });

    it('should not render save button if send link missing', () => {
      bescheidService.getBescheidDraft.mockReturnValue(
        of(createStateResource(createBescheidResource([BescheidLinkRel.BESCHEIDEN]))),
      );

      component.ngOnInit();
      fixture.detectChanges();

      const element = getElementFromFixture(fixture, saveAndSendButtonTestId);
      expect(element).not.toBeInstanceOf(HTMLElement);
    });

    it('should not render save button if link is missing', () => {
      bescheidService.getBescheidDraft.mockReturnValue(
        of(createStateResource(createBescheidResource([BescheidLinkRel.BESCHEIDEN_UND_SENDEN]))),
      );

      component.ngOnInit();
      fixture.detectChanges();

      const element = getElementFromFixture(fixture, saveAndSendButtonTestId);
      expect(element).not.toBeInstanceOf(HTMLElement);
    });
  });

  describe('ngOnInit', () => {
    const bescheidResource: BescheidResource = createBescheidResource();

    it('should set sendBy in form', () => {
      component.initSendBy = jest.fn();
      bescheidService.getBescheidDraft.mockReturnValue(of(createStateResource(bescheidResource)));

      component.ngOnInit();
      component.bescheidResource$.subscribe();

      expect(component.initSendBy).toHaveBeenCalledWith(bescheidResource);
    });
  });

  describe('initSendBy', () => {
    it('should call form service', () => {
      formService.setSendBy = jest.fn();

      component.initSendBy(createBescheidResource());

      expect(formService.setSendBy).toHaveBeenCalledWith(BescheidSendBy.MANUAL);
    });

    it('should not call form service', () => {
      formService.setSendBy = jest.fn();

      component.initSendBy(createBescheidResource([BescheidLinkRel.BESCHEIDEN_UND_SENDEN]));

      expect(formService.setSendBy).not.toHaveBeenCalled();
    });
  });
});
