/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { ComponentFixture, TestBed } from '@angular/core/testing';
import { createEmptyStateResource, createStateResource } from '@alfa-client/tech-shared';
import { getElementFromFixture, mock } from '@alfa-client/test-utils';
import { VorgangResource, VorgangService } from '@alfa-client/vorgang-shared';
import { getDataTestIdOf } from 'libs/tech-shared/test/data-test';
import { createVorgangResource } from 'libs/vorgang-shared/test/vorgang';
import { MockComponent } from 'ng-mocks';
import { of } from 'rxjs';
import { WiedervorlageBreadcrumbContainerComponent } from './wiedervorlage-breadcrumb-container.component';
import { WiedervorlageBreadcrumbComponent } from './wiedervorlage-breadcrumb/wiedervorlage-breadcrumb.component';

describe('WiedervorlageBreadcrumbContainerComponent', () => {
  let component: WiedervorlageBreadcrumbContainerComponent;
  let fixture: ComponentFixture<WiedervorlageBreadcrumbContainerComponent>;

  const vorgangService = mock(VorgangService);

  const wiedervorlageBreadcrumb: string = getDataTestIdOf('wiedervorlage-breadcrumb');

  beforeEach(async () => {
    await TestBed.configureTestingModule({
      declarations: [
        WiedervorlageBreadcrumbContainerComponent,
        MockComponent(WiedervorlageBreadcrumbComponent),
      ],
      providers: [
        {
          provide: VorgangService,
          useValue: vorgangService,
        },
      ],
    }).compileComponents();
  });

  beforeEach(() => {
    fixture = TestBed.createComponent(WiedervorlageBreadcrumbContainerComponent);
    component = fixture.componentInstance;
    fixture.detectChanges();
  });

  it('should create', () => {
    expect(component).toBeTruthy();
  });

  describe('ngOnChanges', () => {
    it('should call vorgangService to get vorgang', () => {
      component.ngOnChanges();

      expect(vorgangService.getVorgangWithEingang).toHaveBeenCalled();
    });
  });

  describe('breadcrumb', () => {
    it('should show if vorgang loaded', () => {
      component.vorgangStatResource$ = of(createStateResource(createVorgangResource()));

      fixture.detectChanges();
      const element = getElementFromFixture(fixture, wiedervorlageBreadcrumb);

      expect(element).toBeInstanceOf(HTMLElement);
    });

    it('should not show if vorgang not laoded', () => {
      component.vorgangStatResource$ = of(createEmptyStateResource<VorgangResource>());

      fixture.detectChanges();
      const element = getElementFromFixture(fixture, wiedervorlageBreadcrumb);

      expect(element).not.toBeInstanceOf(HTMLElement);
    });
  });
});
