import { isNotNull } from '@alfa-client/tech-shared';
import {
  OrganisationsEinheitResource,
  OrganisationsEinheitService,
  ZUSTAENDIGE_STELLE_SERVICE,
  ZustaendigeStelleService,
} from '@alfa-client/zustaendige-stelle-shared';
import { Component, Inject, Input, OnDestroy, OnInit } from '@angular/core';
import { AbstractControl } from '@angular/forms';
import { ResourceUri, getUrl } from '@ngxp/rest';
import { Observable, tap } from 'rxjs';

@Component({
  selector: 'alfa-search-organisations-einheit-container',
  templateUrl: './search-organisations-einheit-container.component.html',
  providers: [{ provide: ZUSTAENDIGE_STELLE_SERVICE, useClass: OrganisationsEinheitService }],
})
export class SearchOrganisationsEinheitContainerComponent implements OnInit, OnDestroy {
  @Input() public fieldControl: AbstractControl<ResourceUri>;

  public organisationsEinheitResource$: Observable<OrganisationsEinheitResource>;

  constructor(
    @Inject(ZUSTAENDIGE_STELLE_SERVICE)
    private service: ZustaendigeStelleService<OrganisationsEinheitResource>,
  ) {}

  ngOnInit(): void {
    this.organisationsEinheitResource$ = this.getSelectedResult();
  }

  ngOnDestroy(): void {
    this.service.clearSelectedResult();
  }

  getSelectedResult(): Observable<OrganisationsEinheitResource> {
    return this.service
      .getSelectedResult()
      .pipe(tap((organisationsEinheitResource: OrganisationsEinheitResource) => this.handleResult(organisationsEinheitResource)));
  }

  handleResult(organisationsEinheitResource: OrganisationsEinheitResource): void {
    if (isNotNull(organisationsEinheitResource)) {
      this.fieldControl.patchValue(getUrl(organisationsEinheitResource));
    }
  }
}
