package de.ozgcloud.alfa.collaboration;

import static org.springframework.hateoas.server.mvc.WebMvcLinkBuilder.*;

import java.util.Objects;

import org.springframework.boot.autoconfigure.condition.ConditionalOnProperty;
import org.springframework.hateoas.EntityModel;
import org.springframework.hateoas.Link;
import org.springframework.hateoas.LinkRelation;
import org.springframework.hateoas.server.RepresentationModelProcessor;
import org.springframework.stereotype.Component;

import de.ozgcloud.alfa.common.ModelBuilder;
import de.ozgcloud.alfa.common.user.CurrentUserService;
import de.ozgcloud.alfa.common.user.UserRole;
import de.ozgcloud.alfa.vorgang.VorgangWithEingang;
import lombok.RequiredArgsConstructor;

@RequiredArgsConstructor
@Component
@ConditionalOnProperty("ozgcloud.feature.collaboration-enabled")
class CollaborationVorgangProcessor implements RepresentationModelProcessor<EntityModel<VorgangWithEingang>> {

	static final LinkRelation REL_COLLABORATIONS = LinkRelation.of("collaborations");
	static final LinkRelation REL_SEARCH_ORGANISATIONS_EINHEIT = LinkRelation.of("searchOrganisationsEinheit");
	static final LinkRelation REL_SEARCH_FACHSTELLE = LinkRelation.of("searchFachstelle");

	private final CurrentUserService currentUserService;
	private final CollaborationService collaborationService;

	@Override
	public EntityModel<VorgangWithEingang> process(EntityModel<VorgangWithEingang> model) {
		var vorgang = model.getContent();

		if (Objects.isNull(vorgang) || !currentUserService.hasRole(UserRole.VERWALTUNG_USER)) {
			return model;
		}

		return ModelBuilder.fromModel(model)
				.ifMatch(() -> !collaborationService.hasCollaboration(vorgang.getId()))
				.addLinks(buildSearchOrganisationsEinheitLink(), buildSearchFachstelleLink())
				.addLink(linkTo(methodOn(CollaborationController.class).getAllByVorgangId(vorgang.getId())).withRel(REL_COLLABORATIONS))
				.buildModel();
	}

	private Link buildSearchOrganisationsEinheitLink() {
		return linkTo(methodOn(OrganisationsEinheitController.class).search(null)).withRel(
				REL_SEARCH_ORGANISATIONS_EINHEIT);
	}

	private Link buildSearchFachstelleLink() {
		return linkTo(methodOn(FachstelleController.class).search(null)).withRel(REL_SEARCH_FACHSTELLE);
	}
}
