package de.ozgcloud.alfa.collaboration;

import java.util.stream.Stream;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import de.ozgcloud.alfa.common.GrpcUtil;
import de.ozgcloud.zufi.grpc.fachstelle.FachstelleServiceGrpc.FachstelleServiceBlockingStub;
import de.ozgcloud.zufi.grpc.fachstelle.GrpcFachstelleGetRequest;
import de.ozgcloud.zufi.grpc.fachstelle.GrpcFachstelleGetResponse;
import de.ozgcloud.zufi.grpc.fachstelle.GrpcFachstelleSearchRequest;
import net.devh.boot.grpc.client.inject.GrpcClient;

@Service
class FachstelleRemoteService {

	@GrpcClient(GrpcUtil.ZUFI_MANAGER_GRPC_CLIENT)
	private FachstelleServiceBlockingStub serviceStub;

	@Autowired
	private FachstelleMapper fachstelleMapper;

	public Stream<Fachstelle> search(String searchBy) {
		var response = serviceStub.search(buildSearchRequest(searchBy));
		return response.getFachstellenList().stream().map(fachstelleMapper::fromGrpc);
	}

	private GrpcFachstelleSearchRequest buildSearchRequest(String searchBy) {
		return GrpcFachstelleSearchRequest.newBuilder().setSearchBy(searchBy).build();
	}

	public Fachstelle getById(String id) {
		var request = buildGetByIdRequest(id);
		var response = serviceStub.getById(request);
		return getFachstelleFromGetByIdResponse(response);
	}

	GrpcFachstelleGetRequest buildGetByIdRequest(String id) {
		return GrpcFachstelleGetRequest.newBuilder().setId(id).build();
	}

	Fachstelle getFachstelleFromGetByIdResponse(GrpcFachstelleGetResponse response) {
		return fachstelleMapper.fromGrpc(response.getFachstelle());
	}
}
