/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.alfa.common.downloadtoken;

import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.*;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.*;

import java.util.UUID;

import org.apache.commons.lang3.StringUtils;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.springframework.http.MediaType;
import org.springframework.test.web.servlet.MockMvc;
import org.springframework.test.web.servlet.ResultActions;
import org.springframework.test.web.servlet.setup.MockMvcBuilders;

import de.ozgcloud.alfa.common.binaryfile.BinaryFileTestFactory;
import de.ozgcloud.common.test.TestUtils;

class DownloadTokenControllerTest {

	@InjectMocks // NOSONAR
	private DownloadTokenController controller;
	@Mock
	private DownloadTokenService service;

	private MockMvc mockMvc;

	@BeforeEach
	void initTest() {
		mockMvc = MockMvcBuilders.standaloneSetup(controller).build();
	}

	final static String TOKEN = UUID.randomUUID().toString();

	@BeforeEach
	void mock() {
		when(service.createToken(any())).thenReturn(TOKEN);
	}

	@Test
	void shouldGenerateToken() throws Exception {
		callEndpoint(createRequest(BinaryFileTestFactory.ID));

		verify(service).createToken(any());
	}

	@Test
	void shouldGenerateTokenWhenNoFileId() throws Exception {
		callEndpoint(createRequest(StringUtils.EMPTY));

		verify(service).createToken(any());
	}

	@Test
	void shouldHaveReponse() throws Exception {
		callEndpoint(createRequest(BinaryFileTestFactory.ID))
				.andExpect(header().string("Location", "http://localhost" + DownloadTokenController.DOWNLOAD_TOKEN_PATH + "?"
						+ DownloadTokenController.PARAM_TOKEN + "=" + TOKEN));
	}

	private ResultActions callEndpoint(String content) throws Exception {
		return mockMvc.perform(
				post(DownloadTokenController.DOWNLOAD_TOKEN_PATH)
						.contentType(MediaType.APPLICATION_JSON)
						.content(content))
				.andExpect(status().isCreated());
	}

	private String createRequest(String fileId) {
		return TestUtils.loadTextFile("jsonTemplates/downloadTokenRequest.json.tmpl", fileId);
	}

}