/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.alfa.common.file;

import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;

import java.util.stream.Stream;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.InjectMocks;
import org.mockito.Mock;

import de.ozgcloud.alfa.attachment.GrpcGetAttachmentsResponseTestFactory;
import de.ozgcloud.alfa.common.callcontext.ContextService;
import de.ozgcloud.alfa.representation.GrpcGetRepresentationsResponseTestFactory;
import de.ozgcloud.alfa.vorgang.VorgangHeaderTestFactory;
import de.ozgcloud.vorgang.grpc.file.FileServiceGrpc.FileServiceBlockingStub;
import de.ozgcloud.vorgang.grpc.file.GrpcGetAttachmentsRequest;
import de.ozgcloud.vorgang.grpc.file.GrpcGetAttachmentsResponse;
import de.ozgcloud.vorgang.grpc.file.GrpcGetRepresentationsRequest;
import de.ozgcloud.vorgang.grpc.file.GrpcGetRepresentationsResponse;

class OzgFileRemoteServiceTest {

	@InjectMocks
	private OzgFileRemoteService remoteService;
	@Mock
	private FileServiceBlockingStub serviceStub;
	@Mock
	private OzgFileMapper mapper;
	@Mock
	private ContextService contextService;

	@Nested
	class TestGetAttachments {

		private final GrpcGetAttachmentsResponse response = GrpcGetAttachmentsResponseTestFactory.create();

		@BeforeEach
		void mock() {
			when(serviceStub.getAttachments(any())).thenReturn(response);
		}

		@Test
		void shouldCallFileStub() {
			doServiceCall();

			verify(serviceStub).getAttachments(any(GrpcGetAttachmentsRequest.class));
		}

		@Test
		void shouldCallMapper() {
			var ozgFileAsStream = doServiceCall();
			ozgFileAsStream = forceActionsDone(ozgFileAsStream);

			verify(mapper).toFile(GrpcOzgFileTestFactory.create());
		}

		private Stream<OzgFile> forceActionsDone(Stream<OzgFile> stream) {
			return stream.toList().stream();
		}

		private Stream<OzgFile> doServiceCall() {
			return remoteService.getAttachments(VorgangHeaderTestFactory.ID);
		}
	}

	@Nested
	class TestGetRepresentations {

		private final GrpcGetRepresentationsResponse response = GrpcGetRepresentationsResponseTestFactory.create();

		@BeforeEach
		void mock() {
			when(serviceStub.getRepresentations(any())).thenReturn(response);
		}

		@Test
		void shouldCallFileStub() {
			doServiceCall();

			verify(serviceStub).getRepresentations(any(GrpcGetRepresentationsRequest.class));
		}

		@Test
		void shouldCallMapper() {
			var ozgFileAsStream = doServiceCall();
			ozgFileAsStream = forceActionsDone(ozgFileAsStream);

			verify(mapper).toFile(GrpcOzgFileTestFactory.create());
		}

		private Stream<OzgFile> forceActionsDone(Stream<OzgFile> stream) {
			return stream.toList().stream();
		}

		private Stream<OzgFile> doServiceCall() {
			return remoteService.getRepresentations(VorgangHeaderTestFactory.ID);
		}
	}
}
