/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.alfa.common.user;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.ArgumentCaptor;
import org.mockito.Captor;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Spy;

import de.ozgcloud.user.grpc.userprofile.UserProfileServiceGrpc.UserProfileServiceBlockingStub;
import de.ozgcloud.user.userprofile.GrpcGetUserProfileRequest;
import io.grpc.Status;
import io.grpc.StatusRuntimeException;

class UserRemoteServiceTest {

	@Spy
	@InjectMocks
	private UserRemoteService service;
	@Mock
	private UserProfileServiceBlockingStub serviceStub;
	@Mock
	private UserProfileMapper mapper;

	@BeforeEach
	void prepareStub() {
		doReturn(serviceStub).when(service).getUserProfileServiceStub();
	}

	@DisplayName("Find by id")
	@Nested
	class TestFindById {

		@Captor
		private ArgumentCaptor<GrpcGetUserProfileRequest> requestCaptor;

		@BeforeEach
		void mock() {
			when(serviceStub.getById(any())).thenReturn(GrpcUserProfileTestFactory.createGetUserProfileResponse());
			when(mapper.mapFrom(any())).thenReturn(UserProfileTestFactory.create());
		}

		@Test
		void shouldCallRemoteService() {
			service.getById(UserProfileTestFactory.ID);

			verify(serviceStub).getById(requestCaptor.capture());
			assertThat(requestCaptor.getValue().getUserId()).isEqualTo(UserProfileTestFactory.ID.toString());
		}

		@Test
		void shouldCallMapper() {
			service.getById(UserProfileTestFactory.ID);

			verify(mapper).mapFrom(GrpcUserProfileTestFactory.create());
		}

		@Test
		void shouldReturnValue() {
			var userProfile = service.getById(UserProfileTestFactory.ID);

			assertThat(userProfile).usingRecursiveComparison().isEqualTo(UserProfileTestFactory.create());
		}
	}

	@DisplayName("Find by externalId")
	@Nested
	class TestFindByExternalId {
		@Test
		void shouldHandleNotFound() {
			when(serviceStub.getByExternalId(any())).thenThrow(new StatusRuntimeException(Status.NOT_FOUND));

			var result = service.getByExternalId(UserProfileTestFactory.EXTERNAL_ID);

			assertThat(result).isEmpty();
		}
	}
}