package de.ozgcloud.alfa.hint;

import static org.mockito.Mockito.*;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.*;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.*;

import java.util.List;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.springframework.test.web.servlet.MockMvc;
import org.springframework.test.web.servlet.ResultActions;
import org.springframework.test.web.servlet.setup.MockMvcBuilders;

import lombok.SneakyThrows;

class HintControllerTest {

	@InjectMocks
	private HintController controller;
	@Mock
	private HintService service;
	@Mock
	private HintModelAssembler modelAssembler;

	private MockMvc mockMvc;

	@BeforeEach
	void initTest() {
		mockMvc = MockMvcBuilders.standaloneSetup(controller).build();
	}

	@DisplayName("GetAll")
	@Nested
	class TestGetAl {

		private List<Hint> hints = List.of(HintTestFactory.create());

		@BeforeEach
		void mockService() {
			when(service.findHints()).thenReturn(hints);
		}

		@SneakyThrows
		@Test
		void shouldReturnStatusOk() {
			callEndpoint().andExpect(status().isOk());
		}

		@Test
		void shouldCallService() {
			callEndpoint();

			verify(service).findHints();
		}

		@Test
		void shouldCallModelAssembler() {
			callEndpoint();

			verify(modelAssembler).toCollectionModel(hints);
		}

		@SneakyThrows
		private ResultActions callEndpoint() {
			return mockMvc.perform(get(HintController.PATH));
		}
	}
}