/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.alfa.representation;

import static org.mockito.Mockito.*;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.*;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.*;

import java.util.stream.Stream;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Spy;
import org.springframework.hateoas.CollectionModel;
import org.springframework.test.web.servlet.MockMvc;
import org.springframework.test.web.servlet.ResultActions;
import org.springframework.test.web.servlet.setup.MockMvcBuilders;

import de.ozgcloud.alfa.common.binaryfile.BinaryFileModelAssembler;
import de.ozgcloud.alfa.common.file.OzgFile;
import de.ozgcloud.alfa.common.file.OzgFileService;
import de.ozgcloud.alfa.common.file.OzgFileTestFactory;
import de.ozgcloud.alfa.vorgang.VorgangHeaderTestFactory;
import lombok.SneakyThrows;

class RepresentationByVorgangControllerTest {

	@Spy
	@InjectMocks
	private RepresentationByVorgangController controller;
	@Mock
	private BinaryFileModelAssembler modelAssembler;
	@Mock
	private OzgFileService fileService;

	private MockMvc mockMvc;

	@BeforeEach
	void init() {
		mockMvc = MockMvcBuilders.standaloneSetup(controller).build();
	}

	@Nested
	class TestGetRepresentations {

		Stream<OzgFile> representations = Stream.of(OzgFileTestFactory.create());

		@BeforeEach
		void mockFileService() {
			when(fileService.getRepresentations(VorgangHeaderTestFactory.ID)).thenReturn(representations);
			when(modelAssembler.toCollectionModel(representations)).thenReturn(CollectionModel.empty());
		}

		@SneakyThrows
		@DisplayName("should return ok")
		@Test
		void shouldReturnOk() {
			var result = callEndpoint();

			result.andExpect(status().isOk());
		}

		@DisplayName("should call file service")
		@Test
		void shouldCallFileService() {
			callEndpoint();

			verify(fileService).getRepresentations(VorgangHeaderTestFactory.ID);
		}

		@SneakyThrows
		@Test
		void shouldCallModelAssembler() {
			callEndpoint();

			verify(modelAssembler).toCollectionModel(representations);
		}

		@SneakyThrows
		@DisplayName("should return entity")
		@Test
		void shouldReturnEntity() {
			var result = callEndpoint();

			result.andExpect(jsonPath("$.content").isEmpty());
		}

		@SneakyThrows
		private ResultActions callEndpoint() {
			return mockMvc.perform(get(RepresentationByVorgangController.PATH, VorgangHeaderTestFactory.ID));
		}
	}
}